// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// (C) 2001-2023 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


//#################################################################################################################
//mSGDMA controller
//#################################################################################################################
//xxx 
//#################################################################################################################

`timescale 1ns / 1ps
module  master_driver_msgdma(
        input  wire        reset_n         , // Reset, low active
        input  wire        clk             , // System clock, 50MHz by default

        input  wire [4:0]  csr_address     , // CSR slave interface
        input  wire        csr_read        , // CSR slave interface
        input  wire        csr_write       , // CSR slave interface
        output reg  [31:0] csr_readdata    , // CSR slave interface
        input  wire [31:0] csr_writedata   , // CSR slave interface
        output wire        csr_waitrequest , // CSR slave interface

        output reg         csn             , // mSDGMA avalon master interface: chip select, low active
        output reg         wen             , // mSDGMA avalon master interface: write enable, low active
        output reg         oen             , // mSDGMA avalon master interface: read enable, low active
        output reg  [3:0]  be              , // mSDGMA avalon master interface: byte enable, high active
        output reg  [31:0] addr            , // mSDGMA avalon master interface: address
        output reg  [31:0] wdata           , // mSDGMA avalon master interface: write data
        input  wire [31:0] rdata           , // mSDGMA avalon master interface: read data
        input  wire        waitrequest_in  , // mSDGMA avalon master interface: WAITREQUEST, high active

        input  wire [1:0]  irq             , // INTRs from dispatchers after completing a transfer
                                             // irq[0]: irq_write
                                             // irq[1]: irq_read
        input  wire        cal_success     , // EMIF/Memory calibration result
        input  wire        cal_fail        , // EMIF/Memory calibration result
        output wire        reset_out       , // Other control/status signal(s)
        output wire        error_mon       , // Other control/status signal(s)
        output wire        status_mon        // Other control/status signal(s)
        );

//#######################################################################################
// Parameters definitions
//#######################################################################################
//mSGDMA Qsys address parameterization
parameter  LOCAL_DATA_WORDS                     = 32'h00000040;         // EMIF AVMM slave (ctrl_amm_0) width in bytes

parameter  DATA_WIDTH                           = 512;
parameter  PRBS_WIDTH                           = 64;
parameter  MAX_BURST_COUNT                      = 32;

parameter  DUT_MEMORY_BASE_ADDRESS              = 40'h0000000000;       // DUT memory qsys base address
parameter  DUT_MEMORY_TOTAL_SIZE                = 40'h0200000000;       // DUT memory total size, up to 1024G bytes
parameter  DUT_MEMORY_TEST_SIZE                 = 40'h0200000000;       // DUT memory size for test, up to 1024G bytes
parameter  DUT_MEMORY_TEST_OFFSET_ADDRESS       = 40'h0000000000;       // DUT memory address for the test to start with

parameter  DUT_MEMORY_TOTAL_SIZE_DEFAULT        = 40'h0100000000;       // Default total size, 4G bytes
parameter  DUT_MEMORY_TEST_OFFSET_ADDR_DEFAULT  = 40'h0000000000;       // Default offset address, 0

parameter  MSGDMA_QSYS_BASE_ADDRESS             = 32'h00000000;         // mSGDMA module Qsys base address
parameter  MSGDMA_DISP_WRITE_CSR_OFFSET         = 32'h000000C0;         // mSGDMA DISPATCHER_WRITE_CSR offset
parameter  MSGDMA_DISP_WRITE_DESCRIPTOR_OFFSET  = 32'h000000E0;         // mSGDMA DISPATCHER_WRITE_DESCRIPTOR offset
parameter  MSGDMA_DISP_READ_CSR_OFFSET          = 32'h00000080;         // mSGDMA DISPATCHER_READ_CSR offset
parameter  MSGDMA_DISP_READ_DESCRIPTOR_OFFSET   = 32'h000000A0;         // mSGDMA DISPATCHER_READ_DESCRIPTOR offset
parameter  MSGDMA_FREQ_COUNTER_OFFSET           = 32'h00000100;         // mSGDMA FREQUENCY_COUNTER module offset
parameter  MSGDMA_PRBS_PATTERN_GEN_OFFSET       = 32'h00000040;         // mSGDMA PRBS_PATTERN_GENERATOR module offset
parameter  MSGDMA_PRBS_PATTERN_CHK_OFFSET       = 32'h00000000;         // mSGDMA PRBS_PATTERN_CHECKER module offset

//Local parameter settings                      
localparam PRBS_PATTERN_CHECKER_ADDR            = MSGDMA_QSYS_BASE_ADDRESS + MSGDMA_PRBS_PATTERN_CHK_OFFSET;
localparam PRBS_PATTERN_GENERATOR_ADDR          = MSGDMA_QSYS_BASE_ADDRESS + MSGDMA_PRBS_PATTERN_GEN_OFFSET;
localparam DISPATCHER_WRITE_CSR_ADDR            = MSGDMA_QSYS_BASE_ADDRESS + MSGDMA_DISP_WRITE_CSR_OFFSET;
localparam DISPATCHER_WRITE_DESCRIPTOR_ADDR     = MSGDMA_QSYS_BASE_ADDRESS + MSGDMA_DISP_WRITE_DESCRIPTOR_OFFSET;
localparam DISPATCHER_READ_CSR_ADDR             = MSGDMA_QSYS_BASE_ADDRESS + MSGDMA_DISP_READ_CSR_OFFSET;
localparam DISPATCHER_READ_DESCRIPTOR_ADDR      = MSGDMA_QSYS_BASE_ADDRESS + MSGDMA_DISP_READ_DESCRIPTOR_OFFSET;
localparam FREQUENCY_COUNTER_ADDR               = MSGDMA_QSYS_BASE_ADDRESS + MSGDMA_FREQ_COUNTER_OFFSET;

//---------------------------------------------------------------------------------------------------------
localparam DATA_WIDTH_ALIGNMENT_BIT_WIDTH = (DATA_WIDTH == 8  ) ? 0 : // Square root of two of (DATA_WIDTH/8)
                                            (DATA_WIDTH == 16 ) ? 1 : // Square root of two of (DATA_WIDTH/8)
                                            (DATA_WIDTH == 32 ) ? 2 : // Square root of two of (DATA_WIDTH/8)
                                            (DATA_WIDTH == 64 ) ? 3 : // Square root of two of (DATA_WIDTH/8)
                                            (DATA_WIDTH == 128) ? 4 : // Square root of two of (DATA_WIDTH/8)
                                            (DATA_WIDTH == 256) ? 5 : // Square root of two of (DATA_WIDTH/8)
                                            (DATA_WIDTH == 512) ? 6 : // Square root of two of (DATA_WIDTH/8)
                                           /*DATA_WIDTH == 1024*/ 7;  // Square root of two of (DATA_WIDTH/8)

localparam MAX_BURST_ALIGNMENT_BIT_WIDTH  = (MAX_BURST_COUNT == 2  ) ? 1 : // Square root of two of MAX_BURST_COUNT
                                            (MAX_BURST_COUNT == 4  ) ? 2 : // Square root of two of MAX_BURST_COUNT
                                            (MAX_BURST_COUNT == 8  ) ? 3 : // Square root of two of MAX_BURST_COUNT
                                            (MAX_BURST_COUNT == 16 ) ? 4 : // Square root of two of MAX_BURST_COUNT
                                            (MAX_BURST_COUNT == 32 ) ? 5 : // Square root of two of MAX_BURST_COUNT
                                            (MAX_BURST_COUNT == 64 ) ? 6 : // Square root of two of MAX_BURST_COUNT
                                            (MAX_BURST_COUNT == 128) ? 7 : // Square root of two of MAX_BURST_COUNT
                                            (MAX_BURST_COUNT == 256) ? 8 : // Square root of two of MAX_BURST_COUNT
                                            (MAX_BURST_COUNT == 512) ? 9 : // Square root of two of MAX_BURST_COUNT
                                           /*MAX_BURST_COUNT == 1024*/ 10; // Square root of two of MAX_BURST_COUNT

localparam BYTE_ALIGNMENT_BIT_WIDTH = DATA_WIDTH_ALIGNMENT_BIT_WIDTH + MAX_BURST_ALIGNMENT_BIT_WIDTH;

localparam POLYNOMIAL_DEFAULT_VALUE = (PRBS_WIDTH == 8)  ? 8'hB8 :                                       // taps [8, 6, 5, 4]
                                      (PRBS_WIDTH == 16) ? 16'hB400 :                                    // taps [16, 14, 13, 11]
                                      (PRBS_WIDTH == 32) ? 32'hA300_0000 :                               // taps [32, 30, 26, 25]
                                      (PRBS_WIDTH == 64) ? 64'hD800_0000_0000_0000 :                     // taps [64, 63, 61, 60]
                                     /*PRBS_WIDTH == 128*/ 128'hE100_0000_0000_0000_0000_0000_0000_0000; // taps [128, 127, 126, 121]

localparam SEED_DEFAULT_VALUE       = (PRBS_WIDTH == 8)  ? 8'h5A :
                                      (PRBS_WIDTH == 16) ? 16'hFACE :
                                      (PRBS_WIDTH == 32) ? 32'hDEAD_BEEF :
                                      (PRBS_WIDTH == 64) ? 64'hFEED_ABBA_BABE_BEEF :
                                     /*PRBS_WIDTH == 128*/ 128'hCA11_DEAF_ABBA_BABE_B01D_5CA1_AB1E_FACE;

localparam MSGMDA_DESCRIPTOR_LENGTH_MAX = {{(32-BYTE_ALIGNMENT_BIT_WIDTH){1'b1}},   // Up to 4G bytes, but full word alignemnt
                                           {BYTE_ALIGNMENT_BIT_WIDTH{1'b0}}};

localparam MSGMDA_DESCRIPTOR_LENGTH_MIN = {{(31-BYTE_ALIGNMENT_BIT_WIDTH){1'b0}},
                                           1'b1, {BYTE_ALIGNMENT_BIT_WIDTH{1'b0}}}; // Down to tens of bytes, but still full word alignemnt

localparam TEST_CONSTANT_DEFAULT    = 128'h5FAF5FAF5FAF5FAF5FAF5FAF5FAF5FAF;

//mSGDMA controller FSM definition              
localparam MSGDMA_IDLE                          = 40'h0000000001; // mSGDMA IDLE state
localparam MSGDMA_INIT                          = 40'h0000000002; // mSGDMA INITIALIZATION state
                                                                  // TEST SIZE/START ADDRESS/TEST ROUNDS
                                                                  // POLYNOMIAL/SEED UPDATE

localparam MSGDMA_PAT_GEN_PLD_LENGTH_SET        = 40'h0000000004; // mSGDMA pattern generator payload length set
localparam MSGDMA_PAT_GEN_SEED_VALUE_SET        = 40'h0000000008; // mSGDMA pattern generator seed words set
localparam MSGDMA_PAT_GEN_POLYNOMIAL_SET        = 40'h0000000010; // mSGDMA pattern generator polynomial words set
localparam MSGDMA_PAT_GEN_RUNNING_MODE_SET      = 40'h0000000020; // mSGDMA pattern generator running mode set
                                                                  // Error insertion, start control, inifinite mode or not

localparam MSGDMA_DISP_WR_CSR_CTRL_RESET_SET    = 40'h0000000040; // mSGDMA write dispatcher CSR: control for reset issue
localparam MSGDMA_DISP_WR_CSR_CTRL_IR_EN_SET    = 40'h0000000080; // mSGDMA write dispatcher CSR: control for IR enable
localparam MSGDMA_DISP_WR_CSR_STATUS_REG_SET    = 40'h0000000100; // mSGDMA write dispatcher CSR: status register
localparam MSGDMA_DISP_WR_DSLV_WR_LOW_ADDR_SET  = 40'h0000000200; // mSGDMA write dispatcher SLV: write address set
localparam MSGDMA_DISP_WR_DSLV_LENGTH_SET       = 40'h0000000400; // mSGDMA write dispatcher SLV: length set
localparam MSGDMA_DISP_WR_DSLV_BURST_SEQ_SET    = 40'h0000000800; // mSGDMA write dispatcher SLV: write/read burst count and sequence number
localparam MSGDMA_DISP_WR_DSLV_STRIDE_SET       = 40'h0000001000; // mSGDMA write dispatcher SLV: write/read burst count and sequence number
localparam MSGDMA_DISP_WR_DSLV_WR_HIGH_ADDR_SET = 40'h0000002000; // mSGDMA write dispatcher SLV: write address set
localparam MSGDMA_DISP_WR_DSLV_CONTROL_REG_SET  = 40'h0000004000; // mSGDMA write dispatcher SLV: control register set

localparam MSGDMA_WAIT_SINGLE_WR_PROCESS_FINISH = 40'h0000008000; // Wait for single write process done
localparam MSGDMA_SINGLE_WR_PROCESS_FINISH      = 40'h0000010000; // Single write process is done

localparam MSGDMA_PAT_CHK_PLD_LENGTH_SET        = 40'h0000020000; // mSGDMA pattern checker payload length set
localparam MSGDMA_PAT_CHK_SEED_VALUE_SET        = 40'h0000040000; // mSGDMA pattern checker seed words set
localparam MSGDMA_PAT_CHK_POLYNOMIAL_SET        = 40'h0000080000; // mSGDMA pattern checker polynomial words set
localparam MSGDMA_PAT_CHK_ERROR_CLEARANCE       = 40'h0000100000; // mSGDMA pattern checker error clearance
localparam MSGDMA_PAT_CHK_RUNNING_MODE_SET      = 40'h0000200000; // mSGDMA pattern checker running mode set
                                                                        // Seed checker, start control, stop on failure, inifinite mode or not

localparam MSGDMA_DISP_RD_CSR_CTRL_RESET_SET    = 40'h0000400000; // mSGDMA read dispatcher CSR: control for reset issue
localparam MSGDMA_DISP_RD_CSR_CTRL_IR_EN_SET    = 40'h0000800000; // mSGDMA read dispatcher CSR: control for IR enable
localparam MSGDMA_DISP_RD_CSR_STATUS_REG_SET    = 40'h0001000000; // mSGDMA read dispatcher CSR: status register
localparam MSGDMA_DISP_RD_DSLV_RD_LOW_ADDR_SET  = 40'h0002000000; // mSGDMA read dispatcher SLV: read address set (optional)
localparam MSGDMA_DISP_RD_DSLV_LENGTH_SET       = 40'h0004000000; // mSGDMA read dispatcher SLV: length set
localparam MSGDMA_DISP_RD_DSLV_BURST_SEQ_SET    = 40'h0008000000; // mSGDMA read dispatcher SLV: write/read burst count and sequence number
localparam MSGDMA_DISP_RD_DSLV_STRIDE_SET       = 40'h0010000000; // mSGDMA read dispatcher SLV: write/read burst count and sequence number
localparam MSGDMA_DISP_RD_DSLV_RD_HIGH_ADDR_SET = 40'h0020000000; // mSGDMA read dispatcher SLV: read address set (optional)
localparam MSGDMA_DISP_RD_DSLV_CONTROL_REG_SET  = 40'h0040000000; // mSGDMA read dispatcher SLV: control register set

localparam MSGDMA_WAIT_SINGLE_RD_PROCESS_FINISH = 40'h0080000000; // Wait for single read process done
localparam MSGDMA_SINGLE_RD_PROCESS_FINISH      = 40'h0100000000; // Single write process is done

localparam MSGDMA_TEST_ERROR_CHECKOUT           = 40'h0200000000; // Memory test error checkout
localparam MSGDMA_EMIF_USRCLK_FREQ_CHECK        = 40'h0400000000; // EMIF user clock frequency detection
localparam MSGDMA_TEST_TIMEOUT                  = 40'h0800000000; // Memory test timeout
localparam MSGDMA_TEST_DONE                     = 40'h1000000000; // Memory test done

//---------------------------------------------------------------------------------------
// Signal definition
//---------------------------------------------------------------------------------------
reg          csr_read_reg;                          // mSGDMA CSR interface
reg          csr_write_reg;                         // mSGDMA CSR interface
wire         csr_read_pos/* synthesis keep */;      // mSGDMA CSR interface
wire         csr_read_neg/* synthesis keep */;      // mSGDMA CSR interface
wire         csr_write_pos/* synthesis keep */;     // mSGDMA CSR interface
wire         csr_write_neg/* synthesis keep */;     // mSGDMA CSR interface
reg  [4:0]   csr_address_delay/* synthesis keep */; // mSGDMA CSR interface
wire         csr_address_diff/* synthesis keep */;  // mSGDMA CSR interface
wire         csr_wr_rd_b2b/* synthesis keep */;     // mSGDMA CSR interface

wire         irq_read/* synthesis keep */;          // mSGDMA test interrupt monitoring and control
wire         irq_write/* synthesis keep */;         // mSGDMA test interrupt monitoring and control
reg          irq_read_reg1;                         // mSGDMA test interrupt monitoring and control
reg          irq_read_reg2;                         // mSGDMA test interrupt monitoring and control
reg          irq_read_pos;                          // mSGDMA test interrupt monitoring and control
reg          irq_write_reg1;                        // mSGDMA test interrupt monitoring and control
reg          irq_write_reg2;                        // mSGDMA test interrupt monitoring and control
reg          irq_write_pos;                         // mSGDMA test interrupt monitoring and control

reg  [63:0]  mem_base_addr;                         // CSR: Memory base address
reg  [39:0]  mem_total_size;                        // CSR: Total memory size
reg  [39:0]  mem_test_size;                         // CSR: Test memory size
reg  [39:0]  mem_test_offset_addr;                  // CSR: Memory offset address to start testing
reg  [127:0] pattern_polynomial;                    // CSR: POLYNOMIAL for pattern generator or checker
reg  [127:0] pattern_seed_value;                    // CSR: SEED VALUE for pattern generator or checker

reg          test_run;                              // CSR: Test running control, rising edge active
reg          test_run_reg1;                         // CSR: One clock delay of test_run
reg          test_run_reg2;                         // CSR: Two clocks delay of test_run
wire         test_run_pos/* synthesis keep */;      // Posedge of test_run
wire         test_run_neg/* synthesis keep */;      // Negedge of test_run
reg          test_run_valid;                        // CSR: Test running valid control, high active
reg  [1:0]   test_mode;                             // CSR: 2'b00: single read round
                                                    // CSR: 2'b01: single write round
                                                    // CSR: 2'b10: single write and read round
                                                    // CSR: 2'b11: infinite write and read test
reg          test_stop_on_fail;                     // CSR: Test stop-on-failure control
reg  [1:0]   test_mode_valid;                       // CSR: Test mode valid, high active
                                                    // CSR: 2'b00: single read round
                                                    // CSR: 2'b01: single write round
                                                    // CSR: 2'b10: single write and read round
                                                    // CSR: 2'b11: infinite write and read test

reg          issue_reset;                           // CSR: for test control or monitoring
reg          err_ins_set;                           // CSR: for test control or monitoring
reg          err_ins_clr;                           // CSR: for test control or monitoring
reg          err_ins;                               // CSR: for test control or monitoring
reg          err_clr_assert;                        // CSR: for test control or monitoring
reg          err_clr_deassert;                      // CSR: for test control or monitoring
reg          err_clr;                               // CSR: for test control or monitoring
reg          seed_value_recfg;                      // CSR: for test control or monitoring
reg  [127:0] test_constant;                         // CSR: for test control or monitoring
reg  [3:0]   test_pattern;                          // CSR: for test control or monitoring
                                                    // 4'b0000: user defined prbs pattern (64-bit width), default test pattern
                                                    // 4'b0001: user defined constant pattern
                                                    // 4'b0010: walking '0' started from "FFFF_FFFF_FFFF_FFFE"
                                                    // 4'b0011: walking '1' started from "0000_0000_0000_0001"
                                                    // Others:  reserved
reg  [127:0] test_constant_valid;                   // Sampled every time during test initialization
reg  [3:0]   test_pattern_valid;                    // Sampled every time during test initialization

reg          error_status;                          // CSR: for test control or monitoring
wire [31:0]  msgdma_status;                         // CSR: for test control or monitoring

reg  [39:0]  msgdma_fsm_next_state/* synthesis keep */;      // mSGDMA FSM next state
reg  [39:0]  msgdma_fsm_current_state/* synthesis keep */;   // mSGDMA FSM current state
reg          msgdma_fsm_state_cnt_clr/* synthesis keep */;   // mSGDMA FSM state counter clearance
reg  [9:0]   msgdma_fsm_state_cnt/* synthesis keep */;       // mSGDMA FSM state timeout counter
reg          msgdma_fsm_state_cnt_1023/* synthesis keep */;  // Duration long enough for performance calculation
reg          waitrequest_in_reg/* synthesis keep */;         // For avalon bus read/write control, key control signal
wire         waitrequest_in_neg/* synthesis keep */;         // For avalon bus read/write control
wire         waitrequest_in_neg_valid/* synthesis keep */;   // For avalon bus read/write control
reg  [2:0]   msgdma_fsm_rw_cycle_cnt/* synthesis keep */;    // For multiple read/write cycles control during single FSM state

reg          test_timeout/* synthesis keep */;               // For test timeout control
reg          test_in_progress/* synthesis keep */;           // For test in progress indication
reg          mem_test_space_valid/* synthesis keep */;       // For mSGDMA test initialization
reg  [63:0]  mem_test_start_addr_valid/* synthesis keep */;  // For mSGDMA test initialization
reg  [39:0]  mem_total_size_valid/* synthesis keep */;       // For mSGDMA test initialization
reg  [39:0]  mem_test_offset_addr_valid/* synthesis keep */; // For mSGDMA test initialization
reg  [39:0]  mem_test_size_max/* synthesis keep */;          // For mSGDMA test initialization
reg          mem_test_size_oversize/* synthesis keep */;     // For mSGDMA test initialization
reg  [39:0]  mem_test_size_temp/* synthesis keep */;         // For mSGDMA test initialization
reg  [39:0]  mem_test_size_alignment/* synthesis keep */;    // For mSGDMA test initialization
reg  [39:0]  mem_test_size_valid/* synthesis keep */;        // For mSGDMA test initialization
reg  [39:0]  mem_read_size/* synthesis keep */;              // Up to 1024GB
reg  [39:0]  mem_write_size/* synthesis keep */;             // Up to 1024GB
reg          mem_read_continue/* synthesis keep */;          // For test size bigger than 4GB
reg          mem_write_continue/* synthesis keep */;         // For test size bigger than 4GB
reg  [63:0]  mem_read_address/* synthesis keep */;           // mSGDMA read test start address
reg  [63:0]  mem_write_address/* synthesis keep */;          // mSGDMA write test start address
reg  [63:0]  payload_length_counter/* synthesis keep */;     // mSGDMA test payload length
reg  [127:0] pattern_polynomial_valid/* synthesis keep */;   // For test pattern control
reg  [127:0] pattern_seed_value_valid/* synthesis keep */;   // For test pattern control

reg          timer_enable/* synthesis keep */;               // For test statistics
reg          timer_cnt_clr/* synthesis keep */;              // For test statistics
reg  [31:0]  timer_counter/* synthesis keep */;              // For test statistics
reg  [31:0]  timer_tx/* synthesis keep */;                   // For test statistics
reg  [31:0]  timer_tx_rx/* synthesis keep */;                // For test statistics
reg  [31:0]  error_counter/* synthesis keep */;              // For test statistics
reg  [5:0]   test_multi_cycle_cnt/* synthesis keep */;       // For test statistics, both read and write cycles
reg  [31:0]  target_freq/* synthesis keep */;                // For test statistics

//---------------------------------------------------------------------------------------
//Main function body
//---------------------------------------------------------------------------------------
assign reset_out     = issue_reset;                                                        // reset control
assign error_mon     = error_status;                                                       // error status
assign status_mon    = cal_success & test_run_valid & (!test_timeout);                     // running status
assign msgdma_status = {24'h000000,                                                        // Reserved
                        2'b00, test_in_progress, test_timeout,                             // mSGDMA overall status
                        cal_success, cal_fail, error_status, test_run_valid};              // mSGDMA overall status

//---------------------------------------------------------------------------------------
// CSR interface
//---------------------------------------------------------------------------------------
//CSR write control
always @(posedge clk or negedge reset_n)
begin
  if(!reset_n)
    begin
    test_run             <= 1'b1;
    test_mode            <= 2'b11;
    test_stop_on_fail    <= 1'b0;
    issue_reset          <= 1'b0;
    err_ins_set          <= 1'b0;
    err_clr_assert       <= 1'b0;
    seed_value_recfg     <= 1'b0;
    test_pattern         <= 4'b0000;
    test_constant        <= TEST_CONSTANT_DEFAULT;
    mem_base_addr        <= DUT_MEMORY_BASE_ADDRESS;
    mem_total_size       <= DUT_MEMORY_TOTAL_SIZE;
    mem_test_size        <= DUT_MEMORY_TEST_SIZE;
    mem_test_offset_addr <= DUT_MEMORY_TEST_OFFSET_ADDRESS;
    pattern_polynomial   <= POLYNOMIAL_DEFAULT_VALUE;
    pattern_seed_value   <= SEED_DEFAULT_VALUE;
    end
  else if(csr_write == 1'b1)
    begin
      case(csr_address)
      5'b00000: begin
                mem_base_addr[31:0] <= csr_writedata[31:0];
                end
      5'b00001: begin
                mem_base_addr[63:32] <= csr_writedata[31:0];
                end
      5'b00010: begin
                mem_total_size[31:0] <= csr_writedata[31:0];
                end
      5'b00011: begin
                mem_total_size[39:32] <= csr_writedata[7:0];
                end
      5'b00100: begin
                mem_test_size[31:0] <= csr_writedata[31:0];
                end
      5'b00101: begin
                mem_test_size[39:32] <= csr_writedata[7:0];
                end
      5'b00110: begin
                mem_test_offset_addr[31:0] <= csr_writedata[31:0];
                end
      5'b00111: begin
                mem_test_offset_addr[39:32] <= csr_writedata[7:0];
                end
      5'b01000: begin
                pattern_polynomial[31:0] <= csr_writedata[31:0];
                end
      5'b01001: begin
                pattern_polynomial[63:32] <= csr_writedata[31:0];
                end
      5'b01010: begin
                pattern_polynomial[95:64] <= csr_writedata[31:0];
                end
      5'b01011: begin
                pattern_polynomial[127:96] <= csr_writedata[31:0];
                end
      5'b01100: begin
                pattern_seed_value[31:0] <= csr_writedata[31:0];
                end
      5'b01101: begin
                pattern_seed_value[63:32] <= csr_writedata[31:0];
                end
      5'b01110: begin
                pattern_seed_value[95:64] <= csr_writedata[31:0];
                end
      5'b01111: begin
                pattern_seed_value[127:96] <= csr_writedata[31:0];
                end
      5'b10000: begin
                test_run          <= csr_writedata[0];
                test_mode         <= csr_writedata[2:1];
                test_stop_on_fail <= csr_writedata[3];
                issue_reset       <= csr_writedata[4];
                err_ins_set       <= csr_writedata[5];
                err_clr_assert    <= csr_writedata[6];
                seed_value_recfg  <= csr_writedata[7];
                test_pattern      <= csr_writedata[11:8];
                end
      5'b10001,
      5'b10010,
      5'b10011: 
                begin
                test_run             <= test_run;
                test_mode            <= test_mode;
                test_stop_on_fail    <= test_stop_on_fail;
                issue_reset          <= issue_reset;
                err_ins_set          <= err_ins_set;
                err_clr_assert       <= err_clr_assert;
                seed_value_recfg     <= seed_value_recfg;
                test_pattern         <= test_pattern;
                test_constant        <= test_constant;
                mem_base_addr        <= mem_base_addr;
                mem_total_size       <= mem_total_size;
                mem_test_size        <= mem_test_size;
                mem_test_offset_addr <= mem_test_offset_addr;
                pattern_polynomial   <= pattern_polynomial;
                pattern_seed_value   <= pattern_seed_value;
                end
      5'b10100: begin
                test_constant[31:0]  <= csr_writedata[31:0];
                end
      5'b10101: 
                begin
                test_constant[63:32] <= csr_writedata[31:0];
                end
      5'b10110: 
                begin
                test_constant[95:64] <= csr_writedata[31:0];
                end
      5'b10111: 
                begin
                test_constant[127:96] <= csr_writedata[31:0];
                end
      5'b11000,
      5'b11001,
      5'b11010,
      5'b11011,
      5'b11100,
      5'b11101,
      5'b11110,
      5'b11111: begin
                test_run             <= test_run;
                test_mode            <= test_mode;
                test_stop_on_fail    <= test_stop_on_fail;
                issue_reset          <= issue_reset;
                err_ins_set          <= err_ins_set;
                err_clr_assert       <= err_clr_assert;
                seed_value_recfg     <= seed_value_recfg;
                test_pattern         <= test_pattern;
                test_constant        <= test_constant;
                mem_base_addr        <= mem_base_addr;
                mem_total_size       <= mem_total_size;
                mem_test_size        <= mem_test_size;
                mem_test_offset_addr <= mem_test_offset_addr;
                pattern_polynomial   <= pattern_polynomial;
                pattern_seed_value   <= pattern_seed_value;
                end
      default:  begin
                test_run             <= 1'b1;
                test_mode            <= 2'b11;
                test_stop_on_fail    <= 1'b0;
                issue_reset          <= 1'b0;
                err_ins_set          <= 1'b0;
                err_clr_assert       <= 1'b0;
                seed_value_recfg     <= 1'b0;
                test_pattern         <= 4'b0000;
                test_constant        <= TEST_CONSTANT_DEFAULT;
                mem_base_addr        <= DUT_MEMORY_BASE_ADDRESS;
                mem_total_size       <= DUT_MEMORY_TOTAL_SIZE;
                mem_test_size        <= DUT_MEMORY_TEST_SIZE;
                mem_test_offset_addr <= DUT_MEMORY_TEST_OFFSET_ADDRESS;
                pattern_polynomial   <= POLYNOMIAL_DEFAULT_VALUE;
                pattern_seed_value   <= SEED_DEFAULT_VALUE;
                end
      endcase
    end
  else
    begin
    test_run             <= test_run;
    test_mode            <= test_mode;
    test_stop_on_fail    <= test_stop_on_fail;
    issue_reset          <= 1'b0;
    err_ins_set          <= 1'b0;
    err_clr_assert       <= 1'b0;
    seed_value_recfg     <= seed_value_recfg;
    test_pattern         <= test_pattern;
    test_constant        <= test_constant;
    mem_base_addr        <= mem_base_addr;
    mem_total_size       <= mem_total_size;
    mem_test_size        <= mem_test_size;
    mem_test_offset_addr <= mem_test_offset_addr;
    pattern_polynomial   <= pattern_polynomial;
    pattern_seed_value   <= pattern_seed_value;
    end
end

//CSR read control
always @ (posedge clk or negedge reset_n)
begin
   if(!reset_n)
     csr_readdata <= 0;
   else if(csr_read == 1'b1)
     begin
       case(csr_address)
       5'b00000: csr_readdata <= mem_base_addr[31:0];
       5'b00001: csr_readdata <= mem_base_addr[63:32];
       5'b00010: csr_readdata <= mem_total_size[31:0];
       5'b00011: csr_readdata <= {24'h000000, mem_total_size[39:32]};
       5'b00100: csr_readdata <= mem_test_size[31:0];
       5'b00101: csr_readdata <= {24'h000000, mem_test_size[39:32]};
       5'b00110: csr_readdata <= mem_test_offset_addr[31:0];
       5'b00111: csr_readdata <= {24'h000000, mem_test_offset_addr[39:32]};
       5'b01000: csr_readdata <= pattern_polynomial[31:0];
       5'b01001: csr_readdata <= pattern_polynomial[63:32];
       5'b01010: csr_readdata <= pattern_polynomial[95:64];
       5'b01011: csr_readdata <= pattern_polynomial[127:96];
       5'b01100: csr_readdata <= pattern_seed_value[31:0];
       5'b01101: csr_readdata <= pattern_seed_value[63:32];
       5'b01110: csr_readdata <= pattern_seed_value[95:64];
       5'b01111: csr_readdata <= pattern_seed_value[127:96];
       5'b10000: csr_readdata <= {8'h00,                                        // [31:24]: reserved
                                  2'b00,                                        // [23:22]: reserved
                                  test_multi_cycle_cnt,                         // [21:16]: test cycle statistics
                                  2'b00,                                        // [15:14]: reserved
                                  test_timeout,                                 // [13]:     read only
                                  test_in_progress,                             // [12]:     read only
                                  test_pattern_valid,                           // [11:8]:  test pattern definitions
                                  seed_value_recfg,                             // [7]:     read and write
                                  err_clr_assert,                               // [6]:     read and write, narrow pulse
                                  err_ins_set,                                  // [5]:     read and write, narrow pulse
                                  issue_reset,                                  // [4]:     read and write, narrow pulse
                                  test_stop_on_fail,                            // [3]:     stop on failure
                                  test_mode_valid,                              // [2:1]:   read and write
                                  test_run_valid};                              // [0]:     read and write
       5'b10001: csr_readdata <= msgdma_status;
       5'b10010: csr_readdata <= msgdma_fsm_current_state[31:0];                // Current state[31:0]
       5'b10011: csr_readdata <= {24'h000000, msgdma_fsm_current_state[39:32]}; // Current state[39:32]
       5'b10100: csr_readdata <= test_constant_valid[31:0];                     // User-defined constant pattern
       5'b10101: csr_readdata <= test_constant_valid[63:32];                    // User-defined constant pattern
       5'b10110: csr_readdata <= test_constant_valid[95:64];                    // User-defined constant pattern
       5'b10111: csr_readdata <= test_constant_valid[127:96];                   // User-defined constant pattern
       5'b11000: csr_readdata <= target_freq;                                   // For memory test performance calculation if needed
       5'b11001: csr_readdata <= LOCAL_DATA_WORDS;                              // For memory test performance calculation if needed
       5'b11010: csr_readdata <= timer_tx;                                      // For memory test performance calculation if needed
       5'b11011: csr_readdata <= timer_tx_rx;                                   // For memory test performance calculation if needed
       5'b11100: csr_readdata <= error_counter;
       5'b11101: csr_readdata <= 0;
       5'b11110: csr_readdata <= 0;
       5'b11111: csr_readdata <= 0;
       default:  csr_readdata <= 0;
       endcase
     end
   else
     csr_readdata <= csr_readdata;
end

//CSR_WAITREQUEST control and its debug logic
always @(posedge clk or negedge reset_n)
begin
 if(!reset_n)
   csr_read_reg <= 1'b0;
 else
   csr_read_reg <= csr_read;
end

always @(posedge clk or negedge reset_n)
begin
 if(!reset_n)
   csr_write_reg <= 1'b0;
 else
   csr_write_reg <= csr_write;
end

always @(posedge clk or negedge reset_n)
begin
 if(!reset_n)
   csr_address_delay <= 5'h0;
 else
   csr_address_delay <= csr_address;
end

assign csr_read_pos  = (!csr_read_reg) & csr_read;   // Rising edge of rd_n
assign csr_read_neg  = (!csr_read) & csr_read_reg;   // Falling edge of rd_n
assign csr_write_pos = (!csr_write_reg) & csr_write; // Rising edge of wr_n
assign csr_write_neg = (!csr_write) & csr_write_reg; // Falling edge of wr_n

//Back to back read/write access corner cases:
//a. back to back read case
//b. back to back write case
//c. back to back read and write case
//d. back to back write and read case

//Address changed
assign csr_address_diff = !(csr_address_delay == csr_address);

//"back to back read and write" and "back to back write and read"
assign csr_wr_rd_b2b = ((csr_read_pos & csr_write_neg) | (csr_write_pos & csr_read_neg));

//WAITREQUEST output, high valid
assign csr_waitrequest = csr_address_diff | csr_wr_rd_b2b;

//---------------------------------------------------------------------------------------
// mSGDMA IRQs, and test running control
//---------------------------------------------------------------------------------------
//IRQ management:
assign irq_write = irq[0]; // Interrupt from dispatchers after completing read transfer
assign irq_read  = irq[1]; // Interrupt from dispatchers after completing write transfer

//Read IRQ: rising edge active
always @(posedge clk or negedge reset_n)
begin
  if(!reset_n)
    begin
    irq_read_reg1 <= 1'b0;
    irq_read_reg2 <= 1'b1;
    irq_read_pos  <= 1'b0;
    end
  else
    begin
    irq_read_reg1 <= irq_read;
    irq_read_reg2 <= !irq_read_reg1;
    irq_read_pos  <= irq_read_reg1 & irq_read_reg2; // Rising edge detection
    end
end

//Write IRQ: rising edge active
always @(posedge clk or negedge reset_n)
begin
  if(!reset_n)
    begin
    irq_write_reg1 <= 1'b0;
    irq_write_reg2 <= 1'b1;
    irq_write_pos  <= 1'b0;
    end
  else
    begin
    irq_write_reg1 <= irq_write;
    irq_write_reg2 <= !irq_write_reg1;
    irq_write_pos  <= irq_write_reg2 & irq_write_reg1; // Rising edge detection
    end
end

always @ (posedge clk or negedge reset_n)
begin
  if(!reset_n)
    begin
    test_run_reg1 <= 1'b0;
    test_run_reg2 <= 1'b0;
    end
  else
    begin
    test_run_reg1 <= test_run;
    test_run_reg2 <= test_run_reg1;
    end
end

assign test_run_pos = test_run_reg1 & (!test_run_reg2); // Posedge of test_run
assign test_run_neg = test_run_reg2 & (!test_run_reg1); // Negedge of test_run

always @ (posedge clk or negedge reset_n)
begin
  if(!reset_n)
    test_mode_valid <= 2'b11;
  else if(test_run_pos == 1'b1)
    test_mode_valid <= test_mode;
  else
    test_mode_valid <= test_mode_valid;
end

always @ (posedge clk or negedge reset_n)
begin
  if(!reset_n)
    test_run_valid <= 1'b1;
  else if(issue_reset == 1'b1)
    test_run_valid <= 1'b0;
  else if(test_run_neg == 1'b1)
    test_run_valid <= 1'b0;
  else if(test_run_pos == 1'b1)
    test_run_valid <= 1'b1;
  else if((irq_read_pos == 1'b1) && (mem_read_continue == 1'b0))
    begin
      if(test_mode_valid == 2'b00)      // single round memory read-only test
        test_run_valid <= 1'b0;
      else if(test_mode_valid == 2'b10) // single round memory write and read test
        test_run_valid <= 1'b0;
      else if(test_run == 1'b0)         // Test stop for inifinite test
        test_run_valid <= 1'b0;
      else
        test_run_valid <= test_run_valid;
    end
  else if((irq_write_pos == 1'b1) && (mem_write_continue == 1'b0))
    begin
      if(test_mode_valid == 2'b01)      // single round memory write-only test
        test_run_valid <= 1'b0;
      else
        test_run_valid <= test_run_valid;
    end
  else
    test_run_valid <= test_run_valid;
end

//---------------------------------------------------------------------------------------
// mSGDMA FSM for MEMORY TEST
//---------------------------------------------------------------------------------------
//Finite State Machine current and next states
always @(posedge clk or negedge reset_n)
begin
 if(!reset_n)
   msgdma_fsm_current_state <= MSGDMA_IDLE;
 else if(issue_reset == 1'b1)
   msgdma_fsm_current_state <= MSGDMA_IDLE;
 else
   msgdma_fsm_current_state <= msgdma_fsm_next_state;
end

//---------------------------------------------------------------------------------------
//Finite State Machine transformation control
//---------------------------------------------------------------------------------------
always @*
begin
  msgdma_fsm_next_state    = MSGDMA_IDLE;
  msgdma_fsm_state_cnt_clr = 1'b0;
  case(msgdma_fsm_current_state)
  MSGDMA_IDLE:                                                             // IDLE state
    begin
      if((cal_success == 1'b1) && (test_run_valid == 1'b1) && (msgdma_fsm_state_cnt_1023 == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_INIT;
        msgdma_fsm_state_cnt_clr = 1'b1;                                   // Clear FSM state counter
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_IDLE;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_INIT:                                                             // INIT state
    begin                                                                  // Test options configuration
      if(msgdma_fsm_state_cnt_1023 == 1'b1)                                // FSM state timeout
        begin
          if(test_mode_valid == 2'b00)                                     // Read test only
            begin
            msgdma_fsm_next_state    = MSGDMA_PAT_CHK_PLD_LENGTH_SET;      // Switch to pattern checker payload length set state
            msgdma_fsm_state_cnt_clr = 1'b1;
            end
          else
            begin
            msgdma_fsm_next_state    = MSGDMA_PAT_GEN_PLD_LENGTH_SET;
            msgdma_fsm_state_cnt_clr = 1'b1;
            end
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_INIT;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_PAT_GEN_PLD_LENGTH_SET:                                           // Pattern generator payload length set
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd5) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_GEN_SEED_VALUE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_GEN_SEED_VALUE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_GEN_PLD_LENGTH_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_PAT_GEN_SEED_VALUE_SET:                                           // Pattern generator SEED VALUE set
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd3) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_GEN_POLYNOMIAL_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_GEN_POLYNOMIAL_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_GEN_SEED_VALUE_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_PAT_GEN_POLYNOMIAL_SET:                                           // Pattern generator POLYNOMIAL set
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd3) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_GEN_RUNNING_MODE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_GEN_RUNNING_MODE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_GEN_POLYNOMIAL_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_PAT_GEN_RUNNING_MODE_SET:                                         // Pattern generator running mode set
    begin                                                                  // Get ready for pattern generation
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_CSR_CTRL_RESET_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_CSR_CTRL_RESET_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_GEN_RUNNING_MODE_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_WR_CSR_CTRL_RESET_SET:                                       // Write Dispatcher CSR control register reset bit set
    begin                                                                  // Reset can take multiple clock cycles to complete
      if(msgdma_fsm_state_cnt_1023 == 1'b1)                                // FSM state timeout, make sure it can recover from reset
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_CSR_CTRL_IR_EN_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_CSR_CTRL_RESET_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_WR_CSR_CTRL_IR_EN_SET:                                       // Write Dispatcher CSR control register IR EN bit set
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_CSR_STATUS_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_CSR_STATUS_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_CSR_CTRL_IR_EN_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_WR_CSR_STATUS_REG_SET:                                       // Write Dispatcher CSR status register set
    begin                                                                  // Probably optional state
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_WR_LOW_ADDR_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_WR_LOW_ADDR_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_CSR_STATUS_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_WR_DSLV_WR_LOW_ADDR_SET:                                     // Write Dispatcher Descriptor Slave write low address register configuration
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_LENGTH_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_LENGTH_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_WR_LOW_ADDR_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_WR_DSLV_LENGTH_SET:                                          // Write Dispatcher Descriptor Slave payload length register configuration
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_BURST_SEQ_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_BURST_SEQ_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_LENGTH_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_WR_DSLV_BURST_SEQ_SET:                                       // Write Dispatcher Descriptor Slave burst and sequence number register configuration
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_STRIDE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_STRIDE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_BURST_SEQ_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_WR_DSLV_STRIDE_SET:                                          // Write Dispatcher Descriptor Slave stride register configuration
    begin
      if(msgdma_fsm_state_cnt_1023 == 1'b1)                                // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_WR_HIGH_ADDR_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_STRIDE_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_WR_DSLV_WR_HIGH_ADDR_SET:                                    // Write Dispatcher Descriptor Slave write high address register configuration
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_CONTROL_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_CONTROL_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_WR_HIGH_ADDR_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_WR_DSLV_CONTROL_REG_SET:                                     // Write Dispatcher Descriptor Slave control register configuration
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_WAIT_SINGLE_WR_PROCESS_FINISH;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_WAIT_SINGLE_WR_PROCESS_FINISH;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_WR_DSLV_CONTROL_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_WAIT_SINGLE_WR_PROCESS_FINISH:                                    // Wait for write process to be done
    begin
      if(irq_write_pos == 1'b1)                                            // IRQ for write done
        begin
        msgdma_fsm_next_state    = MSGDMA_SINGLE_WR_PROCESS_FINISH;
        msgdma_fsm_state_cnt_clr = 1'b1;                                   // Clear FSM state counter
        end
      else if(timer_counter >= 32'h0F000000)                               // Test Timeout, >5s
        begin
        msgdma_fsm_next_state    = MSGDMA_TEST_TIMEOUT;                    // Test QUIT
        msgdma_fsm_state_cnt_clr = 1'b1;                                   // Clear FSM state counter
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_WAIT_SINGLE_WR_PROCESS_FINISH;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_SINGLE_WR_PROCESS_FINISH:                                         // Single write process is DONE
    begin
      if(msgdma_fsm_state_cnt_1023 == 1'b1)                                // FSM state timeout
        begin
          msgdma_fsm_state_cnt_clr = 1'b1;
          if(mem_write_continue == 1'b1)                                   // Continue to cover remaining memory space
            msgdma_fsm_next_state = MSGDMA_DISP_WR_CSR_CTRL_RESET_SET;     // Reconfigure write dispatcher for new write process
          else if(test_mode_valid == 2'b01)                                // If only write memory test is enabled
            msgdma_fsm_next_state = MSGDMA_EMIF_USRCLK_FREQ_CHECK;         // Switch to EMIF user clock frequency check state
          else
            msgdma_fsm_next_state = MSGDMA_PAT_CHK_PLD_LENGTH_SET;         // Switch to memory read and pattern check process
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_SINGLE_WR_PROCESS_FINISH;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_PAT_CHK_PLD_LENGTH_SET:                                           // Pattern checker payload length set
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd5) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_SEED_VALUE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_SEED_VALUE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_PLD_LENGTH_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_PAT_CHK_SEED_VALUE_SET:                                           // Pattern checker SEED VALUE set
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd3) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_POLYNOMIAL_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                           // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_POLYNOMIAL_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_SEED_VALUE_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_PAT_CHK_POLYNOMIAL_SET:                                           // Pattern checker POLYNOMIAL set
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd3) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_ERROR_CLEARANCE;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_ERROR_CLEARANCE;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_POLYNOMIAL_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_PAT_CHK_ERROR_CLEARANCE:                                         // Pattern checker error clearance
    begin
      if(msgdma_fsm_state_cnt_1023 == 1'b1)                               // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_RUNNING_MODE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_ERROR_CLEARANCE;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_PAT_CHK_RUNNING_MODE_SET:                                        // Pattern checker running mode set
    begin                                                                 // Get ready for pattern checkout
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_CSR_CTRL_RESET_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_CSR_CTRL_RESET_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_PAT_CHK_RUNNING_MODE_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_RD_CSR_CTRL_RESET_SET:                                      // Read Dispatcher CSR control register reset bit set
    begin                                                                 // Reset can take multiple clock cycles to complete
      if(msgdma_fsm_state_cnt_1023 == 1'b1)                               // FSM state timeout, make sure it can recover from reset
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_CSR_CTRL_IR_EN_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_CSR_CTRL_RESET_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_RD_CSR_CTRL_IR_EN_SET:                                      // Read Dispatcher CSR control register IR EN bit set
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_CSR_STATUS_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_CSR_STATUS_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_CSR_CTRL_IR_EN_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_RD_CSR_STATUS_REG_SET:                                      // Read Dispatcher CSR status register set
    begin                                                                 // Probably optional state
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_RD_LOW_ADDR_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_RD_LOW_ADDR_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_CSR_STATUS_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_RD_DSLV_RD_LOW_ADDR_SET:                                    // Read Dispatcher Descriptor Slave read low address register configuration
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_LENGTH_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_LENGTH_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_RD_LOW_ADDR_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_RD_DSLV_LENGTH_SET:                                         // Read Dispatcher Descriptor Slave payload length register configuration
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_BURST_SEQ_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_BURST_SEQ_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_LENGTH_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_RD_DSLV_BURST_SEQ_SET:                                      // Read Dispatcher Descriptor Slave burst and sequence register configuration
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_STRIDE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_STRIDE_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_BURST_SEQ_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_RD_DSLV_STRIDE_SET:                                         // Read Dispatcher Descriptor Slave stride register configuration
    begin
      if(msgdma_fsm_state_cnt_1023 == 1'b1)                               // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_RD_HIGH_ADDR_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_STRIDE_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_RD_DSLV_RD_HIGH_ADDR_SET:                                   // Read Dispatcher Descriptor Slave read high address register configuration
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_CONTROL_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_CONTROL_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_RD_HIGH_ADDR_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_DISP_RD_DSLV_CONTROL_REG_SET:                                    // Read Dispatcher Descriptor Slave control register configuration
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_WAIT_SINGLE_RD_PROCESS_FINISH;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_WAIT_SINGLE_RD_PROCESS_FINISH;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_DISP_RD_DSLV_CONTROL_REG_SET;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_WAIT_SINGLE_RD_PROCESS_FINISH:                                   // Wait for read process to be done
    begin
      if(irq_read_pos == 1'b1)                                            // IRQ for write done
        begin
        msgdma_fsm_next_state    = MSGDMA_SINGLE_RD_PROCESS_FINISH;
        msgdma_fsm_state_cnt_clr = 1'b1;                                  // Clear FSM state counter
        end
      else if(timer_counter >= 32'h1E000000)                              // Test Timeout, > 10s
        begin
        msgdma_fsm_next_state    = MSGDMA_TEST_TIMEOUT;                   // Test QUIT
        msgdma_fsm_state_cnt_clr = 1'b1;                                  // Clear FSM state counter
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_WAIT_SINGLE_RD_PROCESS_FINISH;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_SINGLE_RD_PROCESS_FINISH:
    begin
      if(msgdma_fsm_state_cnt_1023 == 1'b1)                               // FSM state timeout
        begin
          msgdma_fsm_state_cnt_clr = 1'b1;
          if(mem_read_continue == 1'b1)                                   // Continue to cover remaining memory space
            msgdma_fsm_next_state = MSGDMA_DISP_RD_CSR_CTRL_RESET_SET;    // Reconfigure read dispatcher for new read process
          else
            msgdma_fsm_next_state = MSGDMA_TEST_ERROR_CHECKOUT;           // Switch to test error check state
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_SINGLE_RD_PROCESS_FINISH;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_TEST_ERROR_CHECKOUT:
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_EMIF_USRCLK_FREQ_CHECK;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_EMIF_USRCLK_FREQ_CHECK;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_TEST_ERROR_CHECKOUT;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_EMIF_USRCLK_FREQ_CHECK:
    begin
      if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
        begin
        msgdma_fsm_next_state    = MSGDMA_TEST_DONE;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else if(msgdma_fsm_state_cnt_1023 == 1'b1)                          // FSM state timeout
        begin
        msgdma_fsm_next_state    = MSGDMA_TEST_DONE;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_EMIF_USRCLK_FREQ_CHECK;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  MSGDMA_TEST_TIMEOUT:
    begin
    msgdma_fsm_next_state    = (test_run_valid == 1'b0) ? MSGDMA_IDLE : MSGDMA_TEST_TIMEOUT; // Stay as it is until test is stopped
    msgdma_fsm_state_cnt_clr = 1'b0;
    end
  MSGDMA_TEST_DONE:
    begin
      if(msgdma_fsm_state_cnt_1023 == 1'b1)
        begin
        msgdma_fsm_next_state    = MSGDMA_IDLE;
        msgdma_fsm_state_cnt_clr = 1'b1;
        end
      else
        begin
        msgdma_fsm_next_state    = MSGDMA_TEST_DONE;
        msgdma_fsm_state_cnt_clr = 1'b0;
        end
    end
  default: 
    begin
    msgdma_fsm_next_state    = MSGDMA_IDLE;
    msgdma_fsm_state_cnt_clr = 1'b0;
    end
  endcase
end

//---------------------------------------------------------------------------------------
//mSGDMA FSM transfer counter control
always @ (posedge clk or negedge reset_n)
begin
  if(!reset_n)
    msgdma_fsm_state_cnt <= 10'd0;
  else if(issue_reset == 1'b1)
    msgdma_fsm_state_cnt <= 10'd0;
  else if(msgdma_fsm_state_cnt_clr == 1'b1)
    msgdma_fsm_state_cnt <= 10'd0;
  else
    msgdma_fsm_state_cnt <= msgdma_fsm_state_cnt + 1'b1;
end

always @ (posedge clk or negedge reset_n)
begin
  if(!reset_n)
    msgdma_fsm_state_cnt_1023 <= 1'b0;
  else if(msgdma_fsm_state_cnt == 10'd1022)
    msgdma_fsm_state_cnt_1023 <= 1'b1;
  else
    msgdma_fsm_state_cnt_1023 <= 1'b0;
end

//WAITREQUEST monitoring
always @ (posedge clk or negedge reset_n)
begin
  if(!reset_n)
    waitrequest_in_reg <= 1'b0;
  else
    waitrequest_in_reg <= waitrequest_in;
end

assign waitrequest_in_neg       = (!waitrequest_in) & waitrequest_in_reg; // Negedge of waitrequest_in
assign waitrequest_in_neg_valid = (waitrequest_in_neg == 1'b1) && 
                                  (csn == 1'b0) && 
                                  ((wen == 1'b0) || (oen == 1'b0));       // Valid negedge of waitrequest_in
                                                                          // Read or write operation done

//Read/write cycles monitoring per FSM state
always @ (posedge clk or negedge reset_n)
begin
  if(!reset_n)
    msgdma_fsm_rw_cycle_cnt <= 3'd0;
  else if(issue_reset == 1'b1)
    msgdma_fsm_rw_cycle_cnt <= 3'd0;
  else if(msgdma_fsm_state_cnt_clr == 1'b1)
    msgdma_fsm_rw_cycle_cnt <= 3'd0;
  else if(waitrequest_in_neg_valid == 1'b1)
    msgdma_fsm_rw_cycle_cnt <= msgdma_fsm_rw_cycle_cnt + 1'b1;
  else
    msgdma_fsm_rw_cycle_cnt <= msgdma_fsm_rw_cycle_cnt;
end

//---------------------------------------------------------------------------------------
//Finite State Machine transformation control
//---------------------------------------------------------------------------------------
always @(posedge clk or negedge reset_n)
begin
 if(!reset_n)
   begin
   csn                        <= 1'b1;
   wen                        <= 1'b1;
   oen                        <= 1'b1;
   be                         <= 0;
   addr                       <= 0;
   wdata                      <= 0;
   test_timeout               <= 1'b0;
   timer_enable               <= 1'b0;
   timer_cnt_clr              <= 1'b0;
   timer_tx                   <= 0;
   timer_tx_rx                <= 0;
   err_ins_clr                <= 1'b0;
   err_clr_deassert           <= 1'b0;
   target_freq                <= 0;
   error_counter              <= 0;
   test_multi_cycle_cnt       <= 0;
   test_in_progress           <= 1'b0;
   mem_test_space_valid       <= 1'b0;
   mem_total_size_valid       <= 0;
   mem_test_offset_addr_valid <= 0;
   mem_test_start_addr_valid  <= 0;
   mem_test_size_max          <= 0;
   mem_test_size_oversize     <= 1'b0;
   mem_test_size_temp         <= 0;
   mem_test_size_alignment    <= 0;
   mem_test_size_valid        <= 0;
   mem_read_size              <= 0;
   mem_write_size             <= 0;
   mem_read_address           <= 0;
   mem_write_address          <= 0;
   payload_length_counter     <= 0;
   mem_read_continue          <= 1'b0;
   mem_write_continue         <= 1'b0;
   test_pattern_valid         <= 0;
   test_constant_valid        <= TEST_CONSTANT_DEFAULT;
   pattern_polynomial_valid   <= POLYNOMIAL_DEFAULT_VALUE;
   pattern_seed_value_valid   <= SEED_DEFAULT_VALUE;
   end
 else if(issue_reset == 1'b1)
   begin
   csn                        <= 1'b1;
   wen                        <= 1'b1;
   oen                        <= 1'b1;
   be                         <= 0;
   addr                       <= 0;
   wdata                      <= 0;
   test_timeout               <= 1'b0;
   timer_enable               <= 1'b0;
   timer_cnt_clr              <= 1'b0;
   timer_tx                   <= 0;
   timer_tx_rx                <= 0;
   err_ins_clr                <= 1'b0;
   err_clr_deassert           <= 1'b0;
   target_freq                <= 0;
   error_counter              <= 0;
   test_multi_cycle_cnt       <= 0;
   test_in_progress           <= 1'b0;
   mem_test_space_valid       <= 1'b0;
   mem_total_size_valid       <= 0;
   mem_test_offset_addr_valid <= 0;
   mem_test_start_addr_valid  <= 0;
   mem_test_size_max          <= 0;
   mem_test_size_oversize     <= 1'b0;
   mem_test_size_temp         <= 0;
   mem_test_size_alignment    <= 0;
   mem_test_size_valid        <= 0;
   mem_read_size              <= 0;
   mem_write_size             <= 0;
   mem_read_address           <= 0;
   mem_write_address          <= 0;
   payload_length_counter     <= 0;
   mem_read_continue          <= 1'b0;
   mem_write_continue         <= 1'b0;
   test_pattern_valid         <= 0;
   test_constant_valid        <= TEST_CONSTANT_DEFAULT;
   pattern_polynomial_valid   <= POLYNOMIAL_DEFAULT_VALUE;
   pattern_seed_value_valid   <= SEED_DEFAULT_VALUE;
   end
 else
   begin
     case(msgdma_fsm_current_state)
     MSGDMA_IDLE: 
       begin
       csn                        <= 1'b1;
       wen                        <= 1'b1;
       oen                        <= 1'b1;
       be                         <= 0;
       addr                       <= 0;
       wdata                      <= 0;
       test_timeout               <= 1'b0;
       timer_enable               <= 1'b0;
       timer_cnt_clr              <= 1'b0;
       timer_tx                   <= timer_tx;
       timer_tx_rx                <= timer_tx_rx;
       err_ins_clr                <= 1'b0;
       err_clr_deassert           <= 1'b0;
       target_freq                <= target_freq;
       error_counter              <= error_counter;
       test_multi_cycle_cnt       <= test_multi_cycle_cnt;
       test_in_progress           <= 1'b0;
       mem_test_space_valid       <= 1'b0;
       mem_total_size_valid       <= mem_total_size_valid;
       mem_test_offset_addr_valid <= mem_test_offset_addr_valid;
       mem_test_start_addr_valid  <= mem_test_start_addr_valid;
       mem_test_size_max          <= mem_test_size_max;
       mem_test_size_oversize     <= mem_test_size_oversize;
       mem_test_size_temp         <= mem_test_size_temp;
       mem_test_size_alignment    <= mem_test_size_alignment;
       mem_test_size_valid        <= mem_test_size_valid;
       mem_read_size              <= mem_read_size;
       mem_write_size             <= mem_write_size;
       mem_read_address           <= mem_read_address;
       mem_write_address          <= mem_write_address;
       payload_length_counter     <= payload_length_counter;
       mem_read_continue          <= mem_read_continue;
       mem_write_continue         <= mem_write_continue;
       test_pattern_valid         <= test_pattern_valid;
       test_constant_valid        <= test_constant_valid;
       pattern_polynomial_valid   <= pattern_polynomial_valid;
       pattern_seed_value_valid   <= pattern_seed_value_valid;
       end
     MSGDMA_INIT:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)
           begin
           timer_cnt_clr            <= 1'b1;                        // Clear timer counter right before new test round
           test_in_progress         <= 1'b1;                        // Test start
           error_counter            <= err_clr ? 0 : error_counter; // Clear total error statistics
           test_multi_cycle_cnt     <= 6'd0;                        // Clear test cycle statistics: both write and read cycles
           test_pattern_valid       <= test_pattern;                // Test pattern
           test_constant_valid      <= test_constant;               // Test constant
           pattern_polynomial_valid <= pattern_polynomial;          // PATTERN_POLYNOMIAL + 1'b1
           pattern_seed_value_valid <= pattern_seed_value;          // PATTERN_SEED_VALUE
           end
         else if(msgdma_fsm_state_cnt == 10'd1)
           begin
           timer_cnt_clr        <= 1'b0;
           err_clr_deassert     <= 1'b1;
           mem_test_space_valid <= (mem_total_size >= mem_test_offset_addr + MSGMDA_DESCRIPTOR_LENGTH_MIN) ? 1'b1 : 1'b0;
           end
         else if(msgdma_fsm_state_cnt == 10'd2)
           begin
             err_clr_deassert <= 1'b0;
             //--------------------------------------------------
             if(mem_test_space_valid == 1'b1)
               begin
               mem_total_size_valid       <= mem_total_size;
               mem_test_offset_addr_valid <= mem_test_offset_addr;
               end
             else
               begin
               mem_total_size_valid       <= DUT_MEMORY_TOTAL_SIZE_DEFAULT;
               mem_test_offset_addr_valid <= DUT_MEMORY_TEST_OFFSET_ADDR_DEFAULT;
               end
           end
         else if(msgdma_fsm_state_cnt == 10'd3)
           begin
           mem_test_size_max          <= mem_total_size_valid - mem_test_offset_addr_valid;
           mem_test_start_addr_valid  <= mem_base_addr + mem_test_offset_addr_valid; // Base address + offset address
           end
         else if(msgdma_fsm_state_cnt == 10'd4)
           begin
           mem_read_address       <= {mem_test_start_addr_valid[63: BYTE_ALIGNMENT_BIT_WIDTH], {BYTE_ALIGNMENT_BIT_WIDTH{1'b0}}};
           mem_write_address      <= {mem_test_start_addr_valid[63: BYTE_ALIGNMENT_BIT_WIDTH], {BYTE_ALIGNMENT_BIT_WIDTH{1'b0}}};
           mem_test_size_oversize <= (mem_test_size > mem_test_size_max) ? 1'b1 : 1'b0;
           end
         else if(msgdma_fsm_state_cnt == 10'd5)
           begin
           mem_test_size_temp     <= (mem_test_size_oversize == 1'b1) ? mem_test_size_max : mem_test_size;
           end
         else if(msgdma_fsm_state_cnt == 10'd6)
           begin
           mem_test_size_alignment <= {mem_test_size_temp[39: BYTE_ALIGNMENT_BIT_WIDTH], {BYTE_ALIGNMENT_BIT_WIDTH{1'b0}}};
           end
         else if(msgdma_fsm_state_cnt == 10'd7)
           begin
           mem_test_size_valid <= mem_test_size_alignment;
           end
         else if(msgdma_fsm_state_cnt == 10'd8)
           begin
           mem_read_size          <= mem_test_size_valid;
           mem_write_size         <= mem_test_size_valid;
           payload_length_counter <= mem_test_size_valid;
           end
         else
           begin
           timer_cnt_clr              <= timer_cnt_clr;
           test_in_progress           <= test_in_progress;
           error_counter              <= error_counter;
           err_clr_deassert           <= err_clr_deassert;
           mem_test_space_valid       <= mem_test_space_valid;
           mem_total_size_valid       <= mem_total_size_valid;
           mem_test_offset_addr_valid <= mem_test_offset_addr_valid;
           mem_test_start_addr_valid  <= mem_test_start_addr_valid;
           mem_test_size_max          <= mem_test_size_max;
           mem_test_size_oversize     <= mem_test_size_oversize;
           mem_test_size_temp         <= mem_test_size_temp;
           mem_test_size_alignment    <= mem_test_size_alignment;
           mem_test_size_valid        <= mem_test_size_valid;
           mem_read_size              <= mem_read_size;
           mem_write_size             <= mem_write_size;
           mem_read_address           <= mem_read_address;
           mem_write_address          <= mem_write_address;
           payload_length_counter     <= payload_length_counter;
           mem_read_continue          <= mem_read_continue;
           mem_write_continue         <= mem_write_continue;
           test_pattern_valid         <= test_pattern_valid;
           test_constant_valid        <= test_constant_valid;
           pattern_polynomial_valid   <= pattern_polynomial_valid;
           pattern_seed_value_valid   <= pattern_seed_value_valid;
           end
       end
     MSGDMA_PAT_GEN_PLD_LENGTH_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR;               // PAYLOAD_LENGTH_COUNTER[31:0]
           wdata <= payload_length_counter[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 4;           // PAYLOAD_LENGTH_COUNTER[63:32]
           wdata <= payload_length_counter[63:32];             // Currently upto 1024GB, so only [39:32] here
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 52;          // Test CONSTANT WORD #1
           wdata <= test_constant_valid[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd2) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 56;          // Test CONSTANT WORD #2
           wdata <= test_constant_valid[63:32];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd3) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 60;          // Test CONSTANT WORD #3
           wdata <= test_constant_valid[95:64];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd4) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 64;          // Test CONSTANT WORD #4
           wdata <= test_constant_valid[127:96];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd5) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_PAT_GEN_SEED_VALUE_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 36;          // SEED VALUE WORD #1
           wdata <= pattern_seed_value_valid[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 40;          // SEED VALUE WORD #2
           wdata <= pattern_seed_value_valid[63:32];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 44;          // SEED VALUE WORD #3
           wdata <= pattern_seed_value_valid[95:64];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd2) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 48;          // SEED VALUE WORD #4
           wdata <= pattern_seed_value_valid[127:96];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd3) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_PAT_GEN_POLYNOMIAL_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 16;          // POLYNOMIAL WORD #1
           wdata <= pattern_polynomial_valid[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 20;          // POLYNOMIAL WORD #2
           wdata <= pattern_polynomial_valid[63:32];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 24;          // POLYNOMIAL WORD #3
           wdata <= pattern_polynomial_valid[95:64];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd2) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 28;          // POLYNOMIAL WORD #4
           wdata <= pattern_polynomial_valid[127:96];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd3) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_PAT_GEN_RUNNING_MODE_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 8;           // GO command
           wdata <= {20'h00000,                                // [31:12]: reserved
                     test_pattern_valid,                       // [11:8]:  test pattern
                     4'b0000,                                  // [7:4]:   reserved
                     1'b1,                                     // [3]:     start/stop
                     seed_value_recfg,                         // [2]:     seed generator reload
                     err_ins,                                  // [1]:     insert error
                     1'b0                                      // [0]:     infinite test mode
                     };
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn         <= 1'b1;
           wen         <= 1'b1;
           oen         <= 1'b1;
           be          <= 0;
           addr        <= addr;
           wdata       <= 0;
           err_ins_clr <= 1'b1;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn         <= 1'b1;
           wen         <= 1'b1;
           oen         <= 1'b1;
           be          <= 0;
           addr        <= addr;
           wdata       <= 0;
           err_ins_clr <= 1'b1;
           end
         else
           begin
           csn         <= csn;
           wen         <= wen;
           oen         <= oen;
           be          <= be;
           addr        <= addr;
           wdata       <= wdata;
           err_ins_clr <= err_ins_clr;
           end
       end
     MSGDMA_DISP_WR_CSR_CTRL_RESET_SET:
       begin
         err_ins_clr <= 1'b0;
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_WRITE_CSR_ADDR + 4;             // Assert reset to write dispatcher
           wdata <= 32'h00000002;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_WRITE_CSR_ADDR + 4;             // De-assert reset to write dispatcher
           wdata <= 32'h00000000;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn                  <= 1'b1;
           wen                  <= 1'b1;
           oen                  <= 1'b1;
           be                   <= 0;
           addr                 <= addr;
           wdata                <= 0;
           test_multi_cycle_cnt <= test_multi_cycle_cnt + 1'b1; // For test cycle statistics
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_WR_CSR_CTRL_IR_EN_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_WRITE_CSR_ADDR + 4;             // Set IRQ enable
           wdata <= 32'h00000010;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_WR_CSR_STATUS_REG_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_WRITE_CSR_ADDR;                 // Clear all the status
           wdata <= 0;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_WR_DSLV_WR_LOW_ADDR_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_WRITE_DESCRIPTOR_ADDR + 4;      // Write Address[31..0]
           wdata <= mem_write_address[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn                <= 1'b1;
           wen                <= 1'b1;
           oen                <= 1'b1;
           be                 <= 0;
           addr               <= addr;
           wdata              <= 0;
           mem_write_continue <= (mem_write_size > MSGMDA_DESCRIPTOR_LENGTH_MAX) ? 1'b1 : 1'b0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn                <= 1'b1;
           wen                <= 1'b1;
           oen                <= 1'b1;
           be                 <= 0;
           addr               <= addr;
           wdata              <= 0;
           mem_write_continue <= (mem_write_size > MSGMDA_DESCRIPTOR_LENGTH_MAX) ? 1'b1 : 1'b0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_WR_DSLV_LENGTH_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_WRITE_DESCRIPTOR_ADDR + 8;      // Length[31..0] 
           wdata <= mem_write_continue ? MSGMDA_DESCRIPTOR_LENGTH_MAX : mem_write_size[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn            <= 1'b1;
           wen            <= 1'b1;
           oen            <= 1'b1;
           be             <= 0;
           addr           <= addr;
           wdata          <= 0;
           mem_write_size <= (mem_write_continue == 1'b1) ? (mem_write_size - MSGMDA_DESCRIPTOR_LENGTH_MAX) : mem_write_size; // Update mem_write_size for next cycle
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn            <= 1'b1;
           wen            <= 1'b1;
           oen            <= 1'b1;
           be             <= 0;
           addr           <= addr;
           wdata          <= 0;
           mem_write_size <= (mem_write_continue == 1'b1) ? (mem_write_size - MSGMDA_DESCRIPTOR_LENGTH_MAX) : mem_write_size; // Update mem_write_size for next cycle
           end
         else
           begin
           csn            <= csn;
           wen            <= wen;
           oen            <= oen;
           be             <= be;
           addr           <= addr;
           wdata          <= wdata;
           mem_write_size <= mem_write_size;
           end
       end
     MSGDMA_DISP_WR_DSLV_BURST_SEQ_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b0100;                                   // Only for write burst size update
           addr  <= DISPATCHER_WRITE_DESCRIPTOR_ADDR + 12;     // {write_burst[7:0], read_burst[7:0], sequence_number[15:0]}
           wdata <= {MAX_BURST_COUNT, 8'h00, 16'h0000};
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_WR_DSLV_STRIDE_SET:
       begin
     //csn                        <= 1'b1;
     //wen                        <= 1'b1;
     //oen                        <= 1'b1;
     //be                         <= 0;
     //addr                       <= 0;
     //wdata                      <= 0;
       timer_enable               <= timer_enable;
       timer_cnt_clr              <= timer_cnt_clr;
       timer_tx                   <= timer_tx;
       timer_tx_rx                <= timer_tx_rx;
       err_ins_clr                <= err_ins_clr;
       err_clr_deassert           <= err_clr_deassert;
       target_freq                <= target_freq;
       error_counter              <= error_counter;
       test_multi_cycle_cnt       <= test_multi_cycle_cnt;
       test_in_progress           <= test_in_progress;
       mem_test_space_valid       <= mem_test_space_valid;
       mem_total_size_valid       <= mem_total_size_valid;
       mem_test_offset_addr_valid <= mem_test_offset_addr_valid;
       mem_test_start_addr_valid  <= mem_test_start_addr_valid;
       mem_test_size_max          <= mem_test_size_max;
       mem_test_size_oversize     <= mem_test_size_oversize;
       mem_test_size_temp         <= mem_test_size_temp;
       mem_test_size_alignment    <= mem_test_size_alignment;
       mem_test_size_valid        <= mem_test_size_valid;
       mem_read_size              <= mem_read_size;
       mem_write_size             <= mem_write_size;
       mem_read_address           <= mem_read_address;
       mem_write_address          <= mem_write_address;
       mem_read_continue          <= mem_read_continue;
       mem_write_continue         <= mem_write_continue;
       test_pattern_valid         <= test_pattern_valid;
       test_constant_valid        <= test_constant_valid;
       pattern_polynomial_valid   <= pattern_polynomial_valid;
       pattern_seed_value_valid   <= pattern_seed_value_valid;
       end
     MSGDMA_DISP_WR_DSLV_WR_HIGH_ADDR_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_WRITE_DESCRIPTOR_ADDR + 24;     // Write Address[63..32]
           wdata <= mem_write_address[63:32];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn                <= 1'b1;
           wen                <= 1'b1;
           oen                <= 1'b1;
           be                 <= 0;
           addr               <= addr;
           wdata              <= 0;
           mem_write_address  <= (mem_write_continue == 1'b1) ? (mem_write_address + MSGMDA_DESCRIPTOR_LENGTH_MAX) : mem_write_address; // Update mem_write_addr for next cycle
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn                <= 1'b1;
           wen                <= 1'b1;
           oen                <= 1'b1;
           be                 <= 0;
           addr               <= addr;
           wdata              <= 0;
           mem_write_address  <= (mem_write_continue == 1'b1) ? (mem_write_address + MSGMDA_DESCRIPTOR_LENGTH_MAX) : mem_write_address; // Update mem_write_addr for next cycle
           end
         else
           begin
           csn                <= csn;
           wen                <= wen;
           oen                <= oen;
           be                 <= be;
           addr               <= addr;
           wdata              <= wdata;
           mem_write_address  <= mem_write_address;
           mem_write_continue <= mem_write_continue;
           end
       end
     MSGDMA_DISP_WR_DSLV_CONTROL_REG_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_WRITE_DESCRIPTOR_ADDR + 28;     // Control[31..0]
           wdata <= 32'h80004000;                              // Enable "Transfer Complete IRQ Mask" and "Go"
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn          <= 1'b1;
           wen          <= 1'b1;
           oen          <= 1'b1;
           be           <= 0;
           addr         <= addr;
           wdata        <= 0;
           timer_enable <= 1'b1;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn          <= 1'b1;
           wen          <= 1'b1;
           oen          <= 1'b1;
           be           <= 0;
           addr         <= addr;
           wdata        <= 0;
           timer_enable <= 1'b1;
           end
         else
           begin
           csn          <= csn;
           wen          <= wen;
           oen          <= oen;
           be           <= be;
           addr         <= addr;
           wdata        <= wdata;
           timer_enable <= timer_enable;
           end
       end
     MSGDMA_WAIT_SINGLE_WR_PROCESS_FINISH:
       begin
         if(irq_write_pos == 1'b1)                             // IRQ for write done
           timer_enable <= 1'b0;                               // Stop timer counting
         else if(timer_counter >= 32'h0F000000)                // Test Timeout, >5s
           timer_enable <= 1'b0;                               // Stop timer counting
         else
           timer_enable <= timer_enable;
       end
     MSGDMA_SINGLE_WR_PROCESS_FINISH:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)
           timer_tx <= mem_write_continue ? timer_tx : timer_counter; // Update TX timer if write test finishes
         else
           timer_tx <= timer_tx;
       end
     MSGDMA_PAT_CHK_PLD_LENGTH_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR;                 // PAYLOAD_LENGTH_COUNTER[31:0]
           wdata <= payload_length_counter[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 4;             // PAYLOAD_LENGTH_COUNTER[63:32]
           wdata <= payload_length_counter[63:32];             // Currently upto 1024GB, so only [39:32] here
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 52;            // Test CONSTANT WORD #1
           wdata <= test_constant_valid[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd2) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 56;            // Test CONSTANT WORD #2
           wdata <= test_constant_valid[63:32];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd3) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 60;            // Test CONSTANT WORD #3
           wdata <= test_constant_valid[95:64];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd4) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 64;            // Test CONSTANT WORD #4
           wdata <= test_constant_valid[127:96];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd5) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_PAT_CHK_SEED_VALUE_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 36;            // SEED VALUE WORD #1
           wdata <= pattern_seed_value_valid[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 40;            // SEED VALUE WORD #2
           wdata <= pattern_seed_value_valid[63:32];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 44;            // SEED VALUE WORD #3
           wdata <= pattern_seed_value_valid[95:64];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd2) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 48;            // SEED VALUE WORD #4
           wdata <= pattern_seed_value_valid[127:96];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd3) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_PAT_CHK_POLYNOMIAL_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 16;            // POLYNOMIAL WORD #1
           wdata <= pattern_polynomial_valid[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 20;            // POLYNOMIAL WORD #2
           wdata <= pattern_polynomial_valid[63:32];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 24;            // POLYNOMIAL WORD #3
           wdata <= pattern_polynomial_valid[95:64];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd2) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 28;            // POLYNOMIAL WORD #4
           wdata <= pattern_polynomial_valid[127:96];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd3) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_PAT_CHK_ERROR_CLEARANCE:                           // Pattern checker error counter clearance
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 8;             // Assert clear to the error counter in the PRBS checker
           wdata <= 32'h00000010;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 8;             // De-assert clear to the error counter in the PRBS checker
           wdata <= 32'h00000000;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_PAT_CHK_RUNNING_MODE_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 8;             // GO command
           wdata <= {20'h00000,                                // [31:12]: reserved
                     test_pattern_valid,                       // [11:8]:  test pattern
                     2'b00,                                    // [7:6]:   reserved
                     1'b0,                                     // [5]:     reserved for failure detection
                     1'b0,                                     // [4]:     error counter clearance already done in earlier state
                     1'b1,                                     // [3]:     start/stop
                     seed_value_recfg,                         // [2]:     seed checker reload
                     test_stop_on_fail,                        // [1]:     stop on failure
                     1'b0                                      // [0]:     infinite test mode
                     };
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_RD_CSR_CTRL_RESET_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_READ_CSR_ADDR + 4;              // Assert reset to write dispatcher
           wdata <= 32'h00000002;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_READ_CSR_ADDR + 4;              // De-assert reset to write dispatcher
           wdata <= 32'h00000000;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn                  <= 1'b1;
           wen                  <= 1'b1;
           oen                  <= 1'b1;
           be                   <= 0;
           addr                 <= addr;
           wdata                <= 0;
           test_multi_cycle_cnt <= test_multi_cycle_cnt + 1'b1; // For test cycle statistics
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_RD_CSR_CTRL_IR_EN_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_READ_CSR_ADDR + 4;              // Set IRQ enable
           wdata <= 32'h00000010;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_RD_CSR_STATUS_REG_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_READ_CSR_ADDR;                  // Clear all the status
           wdata <= 0;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_RD_DSLV_RD_LOW_ADDR_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_READ_DESCRIPTOR_ADDR;           // Read Address[31..0]
           wdata <= mem_read_address[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn               <= 1'b1;
           wen               <= 1'b1;
           oen               <= 1'b1;
           be                <= 0;
           addr              <= addr;
           wdata             <= 0;
           mem_read_continue <= (mem_read_size > MSGMDA_DESCRIPTOR_LENGTH_MAX) ? 1'b1 : 1'b0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn               <= 1'b1;
           wen               <= 1'b1;
           oen               <= 1'b1;
           be                <= 0;
           addr              <= addr;
           wdata             <= 0;
           mem_read_continue <= (mem_read_size > MSGMDA_DESCRIPTOR_LENGTH_MAX) ? 1'b1 : 1'b0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_RD_DSLV_LENGTH_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_READ_DESCRIPTOR_ADDR + 8;       // Length[31..0] 
           wdata <= mem_read_continue ? MSGMDA_DESCRIPTOR_LENGTH_MAX : mem_read_size[31:0];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn           <= 1'b1;
           wen           <= 1'b1;
           oen           <= 1'b1;
           be            <= 0;
           addr          <= addr;
           wdata         <= 0;
           mem_read_size <= (mem_read_continue == 1'b1) ? (mem_read_size - MSGMDA_DESCRIPTOR_LENGTH_MAX) : mem_read_size;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn           <= 1'b1;
           wen           <= 1'b1;
           oen           <= 1'b1;
           be            <= 0;
           addr          <= addr;
           wdata         <= 0;
           mem_read_size <= (mem_read_continue == 1'b1) ? (mem_read_size - MSGMDA_DESCRIPTOR_LENGTH_MAX) : mem_read_size;
           end
         else
           begin
           csn           <= csn;
           wen           <= wen;
           oen           <= oen;
           be            <= be;
           addr          <= addr;
           wdata         <= wdata;
           mem_read_size <= mem_read_size;
           end
       end
     MSGDMA_DISP_RD_DSLV_BURST_SEQ_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b0100;                                   // Only for read burst size update
           addr  <= DISPATCHER_READ_DESCRIPTOR_ADDR + 12;      // {write_burst[7:0], read_burst[7:0], sequence_number[15:0]}
           wdata <= {8'h00, MAX_BURST_COUNT, 16'h0000};
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn   <= csn;
           wen   <= wen;
           oen   <= oen;
           be    <= be;
           addr  <= addr;
           wdata <= wdata;
           end
       end
     MSGDMA_DISP_RD_DSLV_STRIDE_SET:
       begin
     //csn                        <= 1'b1;
     //wen                        <= 1'b1;
     //oen                        <= 1'b1;
     //be                         <= 0;
     //addr                       <= 0;
     //wdata                      <= 0;
       timer_enable               <= timer_enable;
       timer_cnt_clr              <= timer_cnt_clr;
       timer_tx                   <= timer_tx;
       timer_tx_rx                <= timer_tx_rx;
       err_ins_clr                <= err_ins_clr;
       err_clr_deassert           <= err_clr_deassert;
       target_freq                <= target_freq;
       error_counter              <= error_counter;
       test_multi_cycle_cnt       <= test_multi_cycle_cnt;
       test_in_progress           <= test_in_progress;
       mem_test_space_valid       <= mem_test_space_valid;
       mem_total_size_valid       <= mem_total_size_valid;
       mem_test_offset_addr_valid <= mem_test_offset_addr_valid;
       mem_test_start_addr_valid  <= mem_test_start_addr_valid;
       mem_test_size_max          <= mem_test_size_max;
       mem_test_size_oversize     <= mem_test_size_oversize;
       mem_test_size_temp         <= mem_test_size_temp;
       mem_test_size_alignment    <= mem_test_size_alignment;
       mem_test_size_valid        <= mem_test_size_valid;
       mem_read_size              <= mem_read_size;
       mem_write_size             <= mem_write_size;
       mem_read_address           <= mem_read_address;
       mem_write_address          <= mem_write_address;
       mem_read_continue          <= mem_read_continue;
       mem_write_continue         <= mem_write_continue;
       test_pattern_valid         <= test_pattern_valid;
       test_constant_valid        <= test_constant_valid;
       pattern_polynomial_valid   <= pattern_polynomial_valid;
       pattern_seed_value_valid   <= pattern_seed_value_valid;
       end
     MSGDMA_DISP_RD_DSLV_RD_HIGH_ADDR_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_READ_DESCRIPTOR_ADDR + 20;      // Read Address[63..32]
           wdata <= mem_read_address[63:32];
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn              <= 1'b1;
           wen              <= 1'b1;
           oen              <= 1'b1;
           be               <= 0;
           addr             <= addr;
           wdata            <= 0;
           mem_read_address <= (mem_read_continue == 1'b1) ? (mem_read_address + MSGMDA_DESCRIPTOR_LENGTH_MAX) : mem_read_address; // Update mem_read_addr for next cycle
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn              <= 1'b1;
           wen              <= 1'b1;
           oen              <= 1'b1;
           be               <= 0;
           addr             <= addr;
           wdata            <= 0;
           mem_read_address <= (mem_read_continue == 1'b1) ? (mem_read_address + MSGMDA_DESCRIPTOR_LENGTH_MAX) : mem_read_address; // Update mem_read_addr for next cycle
           end
         else
           begin
           csn               <= csn;
           wen               <= wen;
           oen               <= oen;
           be                <= be;
           addr              <= addr;
           wdata             <= wdata;
           mem_read_address  <= mem_read_address;
           mem_read_continue <= mem_read_continue;
           end
       end
     MSGDMA_DISP_RD_DSLV_CONTROL_REG_SET:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= DISPATCHER_READ_DESCRIPTOR_ADDR + 28;      // Control[31..0]
           wdata <= 32'h80004000;                              // Enable "Transfer Complete IRQ Mask" and "Go"
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn          <= 1'b1;
           wen          <= 1'b1;
           oen          <= 1'b1;
           be           <= 0;
           addr         <= addr;
           wdata        <= 0;
           timer_enable <= 1'b1;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn          <= 1'b1;
           wen          <= 1'b1;
           oen          <= 1'b1;
           be           <= 0;
           addr         <= addr;
           wdata        <= 0;
           timer_enable <= 1'b1;
           end
         else
           begin
           csn          <= csn;
           wen          <= wen;
           oen          <= oen;
           be           <= be;
           addr         <= addr;
           wdata        <= wdata;
           timer_enable <= timer_enable;
           end
       end
     MSGDMA_WAIT_SINGLE_RD_PROCESS_FINISH:
       begin
         if(irq_read_pos == 1'b1)                              // IRQ for read done
           timer_enable <= 1'b0;                               // Stop timer counting
         else if(timer_counter >= 32'h1E000000)                // Test Timeout, >10s
           timer_enable <= 1'b0;                               // Stop timer counting
         else
           timer_enable <= timer_enable;
       end
     MSGDMA_SINGLE_RD_PROCESS_FINISH:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)
           timer_tx_rx <= mem_read_continue ? timer_tx_rx : timer_counter; // Update TX&RX timer if read test finishes
         else
           timer_tx_rx <= timer_tx_rx;
       end
     MSGDMA_TEST_ERROR_CHECKOUT:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First read operation
           begin
           csn  <= 1'b0;
           wen  <= 1'b1;
           oen  <= 1'b0;
           be   <= 4'b1111;
           addr <= PRBS_PATTERN_CHECKER_ADDR + 32;             // Read out error counter in pattern checker
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn           <= 1'b1;
           wen           <= 1'b1;
           oen           <= 1'b1;
           be            <= 0;
           addr          <= addr;
           error_counter <= error_counter + rdata;             // waitrequest_in ? error_counter : error_counter + rdata;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn           <= 1'b1;
           wen           <= 1'b1;
           oen           <= 1'b1;
           be            <= 0;
           addr          <= addr;
           error_counter <= error_counter;
           end
         else
           begin
           csn           <= csn;
           wen           <= wen;
           oen           <= oen;
           be            <= be;
           addr          <= addr;
           error_counter <= error_counter;
           end
       end
     MSGDMA_EMIF_USRCLK_FREQ_CHECK:
       begin
         if(msgdma_fsm_state_cnt == 10'd0)                     // First read operation
           begin
           csn  <= 1'b0;
           wen  <= 1'b1;
           oen  <= 1'b0;
           be   <= 4'b1111;
           addr <= FREQUENCY_COUNTER_ADDR;                     // EMIF USER CLOCK FREQUENCY
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn         <= 1'b1;
           wen         <= 1'b1;
           oen         <= 1'b1;
           be          <= 0;
           addr        <= addr;
           target_freq <= waitrequest_in ? target_freq : rdata;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn         <= 1'b1;
           wen         <= 1'b1;
           oen         <= 1'b1;
           be          <= 0;
           addr        <= addr;
           target_freq <= target_freq;
           end
         else
           begin
           csn         <= csn;
           wen         <= wen;
           oen         <= oen;
           be          <= be;
           addr        <= addr;
           target_freq <= target_freq;
           end
       end
     MSGDMA_TEST_TIMEOUT:                                      // Check error statistics and frequency
       begin                                                   // To improve testability especially when RD timeout happens
         test_timeout <= 1'b1;
         //--------------------------------------------------
         if(msgdma_fsm_state_cnt == 10'd0)                     // First read operation
           begin
           csn  <= 1'b0;
           wen  <= 1'b1;
           oen  <= 1'b0;
           be   <= 4'b1111;
           addr <= PRBS_PATTERN_CHECKER_ADDR + 32;             // Read out error counter in pattern checker
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn              <= 1'b1;
           wen              <= 1'b1;
           oen              <= 1'b1;
           be               <= 0;
           addr             <= addr;
           error_counter    <= test_in_progress ? (error_counter + rdata) : error_counter; // waitrequest_in ? error_counter : error_counter + rdata;
           test_in_progress <= 1'b0;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn  <= 1'b0;
           wen  <= 1'b1;
           oen  <= 1'b0;
           be   <= 4'b1111;
           addr <= FREQUENCY_COUNTER_ADDR;                     // EMIF USER CLOCK FREQUENCY
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd2) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn         <= 1'b1;
           wen         <= 1'b1;
           oen         <= 1'b1;
           be          <= 0;
           addr        <= addr;
           target_freq <= waitrequest_in ? target_freq : rdata;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn              <= 1'b1;
           wen              <= 1'b1;
           oen              <= 1'b1;
           be               <= 0;
           addr             <= addr;
           test_in_progress <= 1'b0;
           error_counter    <= error_counter;
           target_freq      <= target_freq;
           end
         else
           begin
           csn              <= csn;
           wen              <= wen;
           oen              <= oen;
           be               <= be;
           addr             <= addr;
           test_in_progress <= test_in_progress;
           error_counter    <= error_counter;
           target_freq      <= target_freq;
           end
       end
     MSGDMA_TEST_DONE:
       begin
         test_in_progress <= 1'b0;
         //--------------------------------------------------
         if(msgdma_fsm_state_cnt == 10'd0)                     // First write operation
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_GENERATOR_ADDR + 8;           // Stop pattern generator, optional but useful for debug
           wdata <= 0;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd0) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b0;
           wen   <= 1'b0;
           oen   <= 1'b1;
           be    <= 4'b1111;
           addr  <= PRBS_PATTERN_CHECKER_ADDR + 8;             // Stop pattern checker, optional but useful for debug
           wdata <= 0;
           end
         else if((msgdma_fsm_rw_cycle_cnt == 3'd1) && (waitrequest_in_neg_valid == 1'b1))
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else if(msgdma_fsm_state_cnt_1023 == 1'b1)            // FSM state timeout
           begin
           csn   <= 1'b1;
           wen   <= 1'b1;
           oen   <= 1'b1;
           be    <= 0;
           addr  <= addr;
           wdata <= 0;
           end
         else
           begin
           csn  <= csn;
           wen  <= wen;
           oen  <= oen;
           be   <= be;
           addr <= addr;
           end
       end
     default:
       begin
       csn                        <= 1'b1;
       wen                        <= 1'b1;
       oen                        <= 1'b1;
       be                         <= 0;
       addr                       <= 0;
       wdata                      <= 0;
       test_timeout               <= 1'b0;
       timer_enable               <= 1'b0;
       timer_cnt_clr              <= 1'b0;
       timer_tx                   <= 0;
       timer_tx_rx                <= 0;
       err_ins_clr                <= 1'b0;
       err_clr_deassert           <= 1'b0;
       target_freq                <= 0;
       error_counter              <= 0;
       test_multi_cycle_cnt       <= 0;
       test_in_progress           <= 1'b0;
       mem_test_space_valid       <= 1'b0;
       mem_total_size_valid       <= 0;
       mem_test_offset_addr_valid <= 0;
       mem_test_start_addr_valid  <= 0;
       mem_test_size_max          <= 0;
       mem_test_size_oversize     <= 0;
       mem_test_size_temp         <= 0;
       mem_test_size_alignment    <= 0;
       mem_test_size_valid        <= 0;
       mem_read_size              <= 0;
       mem_write_size             <= 0;
       mem_read_address           <= 0;
       mem_write_address          <= 0;
       payload_length_counter     <= 0;
       mem_read_continue          <= 1'b0;
       mem_write_continue         <= 1'b0;
       test_pattern_valid         <= 0;
       test_constant_valid        <= TEST_CONSTANT_DEFAULT;
       pattern_polynomial_valid   <= POLYNOMIAL_DEFAULT_VALUE;
       pattern_seed_value_valid   <= SEED_DEFAULT_VALUE;
       end
     endcase
   end
end

//---------------------------------------------------------------------------------------
//Test timer control
always @ (posedge clk or negedge reset_n)
begin
  if(!reset_n)
    timer_counter <= 0;
  else if((timer_cnt_clr == 1'b1) || (issue_reset == 1'b1))
    timer_counter <= 0;
  else if(timer_enable == 1'b1)
    timer_counter <= timer_counter + 1'b1;
  else
    timer_counter <= timer_counter;
end

//error and timer control
always @(posedge clk or negedge reset_n)
begin
  if(!reset_n)
    err_ins <= 1'b0;
  else if(err_ins_clr == 1'b1)
    err_ins <= 1'b0;	
  else if(err_ins_set == 1'b1)
    err_ins <= 1'b1;
  else
    err_ins <= err_ins;
end

always @(posedge clk or negedge reset_n)
begin
  if(!reset_n)
    err_clr <= 1'b0;
  else if(err_clr_deassert == 1'b1)
    err_clr <= 1'b0;			
  else if(err_clr_assert == 1'b1)
    err_clr <= 1'b1;
  else
    err_clr <= err_clr;
end

always @(posedge clk or negedge reset_n)
begin
  if(!reset_n)
    error_status <= 1'b0;
  else if(err_clr == 1'b1)
    error_status <= 1'b0;
  else if(issue_reset == 1'b1)
    error_status <= 1'b0;
  else if(error_counter != 0)
    error_status <= 1'b1;
  else
    error_status <= 1'b0;
end

endmodule
//---------------------------------------------------------------------------------------
// end of file
//---------------------------------------------------------------------------------------