// (C) 2001-2023 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ns / 100ps

module  bts_xcvr (
        //Reset&Clock
        input  wire        fpga_resetn    , // Reset, low active
        input  wire        clk_fpga_p     , // System clock, 100MHz
        input  wire        qsfpdd_refclk_p  , // QSFPDD reference clock 156.25MHz
		
		//green leds   
        output wire        fpga_led0      ,
		output wire        fpga_led1      ,
		output wire        fpga_led2      ,
		output wire        fpga_led3      ,

        //XCVR channels
        input  wire [7:0]  qsfpdd_rx_p      , 
        input  wire [7:0]  qsfpdd_rx_n      , 
        output wire [7:0]  qsfpdd_tx_p      , 
        output wire [7:0]  qsfpdd_tx_n       
        );

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Signal definitions
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
wire          ninit_done/* synthesis keep */;
wire          S_global_resetn/* synthesis keep */;
reg  [26:0]   heart_beat_cnt/* synthesis keep */;
wire          clk_50m     ; // System clock, 50MHz
wire          sys_pll_locked;



 alt_pll alt_pll_inst0 (
		.rst      (!S_global_resetn),      //   input,  width = 1,   reset.reset
		.refclk   (clk_fpga_p),   //   input,  width = 1,  refclk.clk
		.locked   (sys_pll_locked),   //  output,  width = 1,  locked.export
		.outclk_0 (clk_50m)  //  output,  width = 1, outclk0.clk
	);
	

//----------------------------------------------------------------------------------------------------------
// Intel Agilex Reset Release
//----------------------------------------------------------------------------------------------------------
reset_release reset_release_inst 
              (
              .ninit_done (ninit_done)  // ninit_done.ninit_done
              );

assign S_global_resetn = !ninit_done & fpga_resetn;


assign fpga_led0 = heart_beat_cnt[26];
assign fpga_led1 = sys_pll_locked;
assign fpga_led2 = ninit_done;
assign fpga_led3 = fpga_resetn;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Heart-beat indicate
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
always @(posedge clk_50m or negedge S_global_resetn)
    if (!S_global_resetn)
        heart_beat_cnt <= 27'h0; //0x7FFFFFF
    else
        heart_beat_cnt <= heart_beat_cnt + 1'b1;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//XCVR system Qsys module
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
q_sys   q_sys_u0 (
        .clk_100_clk                                                  (clk_fpga_p       ), //                                      clk_100.clk
        .reset_100_reset_n                                            (S_global_resetn        ), //                                    reset_100.reset_n
        .clk_50_clk                                                   (clk_50m    ), //                                       clk_50.clk
        .reset_50_reset_n                                             (S_global_resetn        ), //                                     reset_50.reset_n
        .systemclk_f_0_refclk_fgt_in_refclk_fgt_6                     (qsfpdd_refclk_p   ), //                     systemclk_f_0_refclk_fgt.in_refclk_fgt_0
        .ftile_xcvr_test_0_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd_tx_p[0]      ), //     ftile_xcvr_test_0_directphy_f_0_tx_serial_data.tx_serial_data
        .ftile_xcvr_test_0_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd_tx_n[0]      ), //   ftile_xcvr_test_0_directphy_f_0_tx_serial_data_n.tx_serial_data_n
        .ftile_xcvr_test_0_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd_rx_p[0]      ), //     ftile_xcvr_test_0_directphy_f_0_rx_serial_data.rx_serial_data
        .ftile_xcvr_test_0_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd_rx_n[0]      ), //   ftile_xcvr_test_0_directphy_f_0_rx_serial_data_n.rx_serial_data_n
        .ftile_xcvr_test_1_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd_tx_p[1]      ), //     ftile_xcvr_test_1_directphy_f_0_tx_serial_data.tx_serial_data
        .ftile_xcvr_test_1_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd_tx_n[1]      ), //   ftile_xcvr_test_1_directphy_f_0_tx_serial_data_n.tx_serial_data_n
        .ftile_xcvr_test_1_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd_rx_p[1]      ), //     ftile_xcvr_test_1_directphy_f_0_rx_serial_data.rx_serial_data
        .ftile_xcvr_test_1_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd_rx_n[1]      ), //   ftile_xcvr_test_1_directphy_f_0_rx_serial_data_n.rx_serial_data_n
        .ftile_xcvr_test_2_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd_tx_p[2]      ), //     ftile_xcvr_test_2_directphy_f_0_tx_serial_data.tx_serial_data
        .ftile_xcvr_test_2_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd_tx_n[2]      ), //   ftile_xcvr_test_2_directphy_f_0_tx_serial_data_n.tx_serial_data_n
        .ftile_xcvr_test_2_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd_rx_p[2]      ), //     ftile_xcvr_test_2_directphy_f_0_rx_serial_data.rx_serial_data
        .ftile_xcvr_test_2_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd_rx_n[2]      ), //   ftile_xcvr_test_2_directphy_f_0_rx_serial_data_n.rx_serial_data_n
        .ftile_xcvr_test_3_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd_tx_p[3]      ), //     ftile_xcvr_test_3_directphy_f_0_tx_serial_data.tx_serial_data
        .ftile_xcvr_test_3_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd_tx_n[3]      ), //   ftile_xcvr_test_3_directphy_f_0_tx_serial_data_n.tx_serial_data_n
        .ftile_xcvr_test_3_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd_rx_p[3]      ), //     ftile_xcvr_test_3_directphy_f_0_rx_serial_data.rx_serial_data
        .ftile_xcvr_test_3_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd_rx_n[3]      ),  //   ftile_xcvr_test_3_directphy_f_0_rx_serial_data_n.rx_serial_data_n
        .ftile_xcvr_test_4_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd_tx_p[4]      ), //     ftile_xcvr_test_4_directphy_f_0_tx_serial_data.tx_serial_data
        .ftile_xcvr_test_4_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd_tx_n[4]      ), //   ftile_xcvr_test_4_directphy_f_0_tx_serial_data_n.tx_serial_data_n
        .ftile_xcvr_test_4_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd_rx_p[4]      ), //     ftile_xcvr_test_4_directphy_f_0_rx_serial_data.rx_serial_data
        .ftile_xcvr_test_4_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd_rx_n[4]      ), //   ftile_xcvr_test_4_directphy_f_0_rx_serial_data_n.rx_serial_data_n
        .ftile_xcvr_test_5_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd_tx_p[5]      ), //     ftile_xcvr_test_5_directphy_f_0_tx_serial_data.tx_serial_data
        .ftile_xcvr_test_5_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd_tx_n[5]      ), //   ftile_xcvr_test_5_directphy_f_0_tx_serial_data_n.tx_serial_data_n
        .ftile_xcvr_test_5_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd_rx_p[5]      ), //     ftile_xcvr_test_5_directphy_f_0_rx_serial_data.rx_serial_data
        .ftile_xcvr_test_5_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd_rx_n[5]      ), //   ftile_xcvr_test_5_directphy_f_0_rx_serial_data_n.rx_serial_data_n
        .ftile_xcvr_test_6_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd_tx_p[6]      ), //     ftile_xcvr_test_6_directphy_f_0_tx_serial_data.tx_serial_data
        .ftile_xcvr_test_6_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd_tx_n[6]      ), //   ftile_xcvr_test_6_directphy_f_0_tx_serial_data_n.tx_serial_data_n
        .ftile_xcvr_test_6_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd_rx_p[6]      ), //     ftile_xcvr_test_6_directphy_f_0_rx_serial_data.rx_serial_data
        .ftile_xcvr_test_6_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd_rx_n[6]      ), //   ftile_xcvr_test_6_directphy_f_0_rx_serial_data_n.rx_serial_data_n
        .ftile_xcvr_test_7_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd_tx_p[7]      ), //     ftile_xcvr_test_7_directphy_f_0_tx_serial_data.tx_serial_data
        .ftile_xcvr_test_7_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd_tx_n[7]      ), //   ftile_xcvr_test_7_directphy_f_0_tx_serial_data_n.tx_serial_data_n
        .ftile_xcvr_test_7_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd_rx_p[7]      ), //     ftile_xcvr_test_7_directphy_f_0_rx_serial_data.rx_serial_data
        .ftile_xcvr_test_7_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd_rx_n[7]      )  //   ftile_xcvr_test_7_directphy_f_0_rx_serial_data_n.rx_serial_data_n
    );

endmodule
