/*****************************************************************************
 * INTEL CONFIDENTIAL
 * Copyright 2024 Intel Corporation.  All Rights Reserved.
 *
 * The source code contained or described herein and all documents related
 * to the source code ("Material") are owned by Intel Corporation or its
 * suppliers or licensors. Title to the Material remains with Intel
 * Corporation or its suppliers and licensors. The Material contains trade
 * secrets and proprietary and confidential information of Intel or its
 * suppliers and licensors. The Material is protected by worldwide copyright
 * and trade secret laws and treaty provisions. No part of the Material may
 * be used, copied, reproduced, modified, published, uploaded, posted,
 * transmitted, distributed, or disclosed in any way without Intel's prior
 * express written permission.
 *
 * No license under any patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or
 * delivery of the Materials, either expressly, by implication, inducement,
 * estoppel or otherwise. Any license under such intellectual property rights
 * must be express and approved by Intel in writing.
 *****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "priv/alt_busy_sleep.h"
#include "config.h"
#include "itl_i2c_interface.h"

ALT_AVALON_I2C_DEV_t *gp_i2c0 = NULL;

/* said not meet i2c spec...*/
void adjust_scl(ALT_AVALON_I2C_DEV_t *dev)
{
    ALT_AVALON_I2C_MASTER_CONFIG_t cfg;
    memset(&cfg, 0,sizeof(cfg));

    alt_avalon_i2c_master_config_get(dev, &cfg);
    cfg.scl_hcnt -= ALT_AVALON_I2C_DIFF_LCNT_HCNT;
    cfg.scl_lcnt += ALT_AVALON_I2C_DIFF_LCNT_HCNT;

    alt_avalon_i2c_master_config_set(dev, &cfg);
}

int itl_i2c_open_devices()
{
    gp_i2c0 = alt_avalon_i2c_open(I2C_0_NAME);
    if(gp_i2c0 == NULL)
    {
        DBG_PRINT("%s open i2c 0 error\n", I2C_0_NAME);
        return -1;
    }
    //adjust_scl(gp_i2c0);

    return 0;
}


ALT_AVALON_I2C_DEV_t* get_i2c_master(int i2c_bus)
{
    ALT_AVALON_I2C_DEV_t* pdev = NULL;

    switch(i2c_bus)
    {
        case 1:
            pdev = gp_i2c0;
            break;

        default:
            break;
    }

    return pdev;
}

/*
 mostly i2c received data with MSB in lower adder, this is big endian order, but nios is little endian
*/
alt_u32 hex2int(alt_u8 *buf, int bitWidth)
{
    alt_u8 data[4];
    memset(data, 0, sizeof(data));

    switch (bitWidth)
    {
        case 8:
            data[0] = buf[0];
            break;
        case 16:
            data[0] = buf[1];
            data[1] = buf[0];
            break;
        case 24:
            data[0] = buf[2];
            data[1] = buf[1];
            data[2] = buf[0];
            break;
        case 32:
            data[0] = buf[3];
            data[1] = buf[2];
            data[2] = buf[1];
            data[3] = buf[0];
            break;
        default:
            break;
    }

    return *(alt_u32 *)data;
}

void int2hex(alt_u32 data, alt_u8 *buf, int bitWidth)
{
    alt_u8 *dataBuf;
    dataBuf = (alt_u8 *)&data;
    memset(buf, 0, 4);

    switch (bitWidth)
    {
        case 8:
            buf[0] = dataBuf[0];
            break;
        case 16:
            buf[0] = dataBuf[1];
            buf[1] = dataBuf[0];
            break;
        case 24:
            buf[0] = dataBuf[2];
            buf[1] = dataBuf[1];
            buf[2] = dataBuf[0];
            break;
        case 32:
            buf[0] = dataBuf[3];
            buf[1] = dataBuf[2];
            buf[2] = dataBuf[1];
            buf[3] = dataBuf[0];
            break;
        default:
            break;
    }

    return;
}

ALT_AVALON_I2C_STATUS_CODE itl_i2c_read_reg(int i2c_bus, alt_u8 slave_addr, alt_u16 reg, alt_u32* data, alt_u8 bitwidth)
{
    ALT_AVALON_I2C_DEV_t* pdev = NULL;
    ALT_AVALON_I2C_STATUS_CODE rt = ALT_AVALON_I2C_SUCCESS;
    alt_u8 rxbuf[4] = {0};
    alt_u8 txbuf[2] = {0};
    int rxsize = 0;
    int txsize = 0;

    pdev = get_i2c_master(i2c_bus);
    if (pdev == NULL)
    {
        DBG_PRINT("I2C bus should be 1 (%d)\n", i2c_bus);
        return ERR_PARAM;
    }

    memset(rxbuf, 0, sizeof(rxbuf));

    alt_avalon_i2c_master_target_set(pdev, slave_addr);

    if(reg > 0xff)
    {
        txbuf[0] = reg;
        txbuf[1] = reg>>8;
        txsize = 2;
    }
    else
    {
        txbuf[0] = reg;
        txsize = 1;
    }

    switch(bitwidth)
    {
        case 16:
            rxsize = 2;
            break;
        case 24:
            rxsize = 3;
            break;
        case 32:
            rxsize = 4;
            break;
        case 8:
        default:
            rxsize = 1;
            break;
    }

    rt = alt_avalon_i2c_master_tx_rx(pdev, txbuf, txsize, rxbuf, rxsize, 0);
    *data = hex2int(rxbuf, bitwidth);
    return rt;
}

ALT_AVALON_I2C_STATUS_CODE itl_i2c_write_reg(int i2c_bus, alt_u8 slave_addr, alt_16 reg, alt_u32 data, alt_u8 bitwidth)
{
    ALT_AVALON_I2C_DEV_t *pdev = NULL;
    ALT_AVALON_I2C_STATUS_CODE rt = ALT_AVALON_I2C_SUCCESS;
    alt_u8 txbuf[6] = {0};
    alt_u32 txsize = 0;
    alt_u32 txDataIdx = 0;
    alt_u32 retry  = 5;

    pdev = get_i2c_master(i2c_bus);
    if (pdev == NULL)
    {
        DBG_PRINT("I2C bus should be 1 (%d)\n", i2c_bus);
        return ERR_PARAM;
    }

    memset(txbuf, 0, sizeof(txbuf));

    alt_avalon_i2c_master_target_set(pdev, slave_addr);

    if(reg > 0xff)
    {
        txbuf[0] = reg;
        txbuf[1] = reg>>8;
        txDataIdx = 2;
    }
    else
    {
        txbuf[0] = reg;
        txDataIdx = 1;
    }

    int2hex(data, &txbuf[txDataIdx], bitwidth);
    switch(bitwidth)
    {
        case 16:
            txsize = txDataIdx + 2;
            break;
        case 24:
            txsize = txDataIdx + 3;
            break;
        case 32:
            txsize = txDataIdx + 4;
            break;
        case 8:
        default:
            txsize = txDataIdx + 1;
            break;
    }

    while (retry--)
    {
        if (retry<2)
            alt_busy_sleep(10000);

        rt = alt_avalon_i2c_master_transmit(pdev, txbuf, txsize, ALT_AVALON_I2C_NO_RESTART, ALT_AVALON_I2C_STOP);
        if ((rt==ALT_AVALON_I2C_ARB_LOST_ERR) || (rt==ALT_AVALON_I2C_NACK_ERR) || (rt==ALT_AVALON_I2C_BUSY))
            continue;
        break;
    }

    return rt;
}
