/*****************************************************************************
 * INTEL CONFIDENTIAL
 * Copyright 2024 Intel Corporation.  All Rights Reserved.
 *
 * The source code contained or described herein and all documents related
 * to the source code ("Material") are owned by Intel Corporation or its
 * suppliers or licensors. Title to the Material remains with Intel
 * Corporation or its suppliers and licensors. The Material contains trade
 * secrets and proprietary and confidential information of Intel or its
 * suppliers and licensors. The Material is protected by worldwide copyright
 * and trade secret laws and treaty provisions. No part of the Material may
 * be used, copied, reproduced, modified, published, uploaded, posted,
 * transmitted, distributed, or disclosed in any way without Intel's prior
 * express written permission.
 *
 * No license under any patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or
 * delivery of the Materials, either expressly, by implication, inducement,
 * estoppel or otherwise. Any license under such intellectual property rights
 * must be express and approved by Intel in writing.
 *****************************************************************************/
 
#ifndef __PCA9534_H__
#define __PCA9534_H__

/* definition in PCA9534 */
#define ITL_PCA9534_REG_IN                 0x0
#define ITL_PCA9534_REG_OUT                0x1
#define ITL_PCA9534_REG_POLARIT            0x2
#define ITL_PCA9534_REG_CONFIG             0x3

//input bit
#define ITL_PCA9534_QSFP_QSFPDD_INTL_BIT    (1)
#define ITL_PCA9534_QSFP_MODPRS_BIT         (4)
#define ITL_PCA9534_QSFPDD_MODPRS_BIT       (5)
//output bit
#define ITL_PCA9534_QSFP_QSFPDD_RESET_BIT   (0)
#define ITL_PCA9534_QSFP_LPMOD_BIT          (2)
#define ITL_PCA9534_QSFPDD_LPMOD_BIT        (3)
#define ITL_PCA9534_QSFP_SEL_BIT            (6)
#define ITL_PCA9534_QSFPDD_SEL_BIT          (7)

//I2C 7-bit address
#define ITL_ADDR_PCA9534            (0x4E>>1)
#define ITL_ADDR_QSFP_QSFPDD        (0xA0>>1)

#define ITL_PCA9534_READ(addr,reg) \
    ret = itl_pca9534_out_read_reg(addr, reg, &data);\
    if(0 == ret)\
        printf("[%s] addr 0x%x reg %d = 0x%lx\n", #reg, addr, reg, data);\
    else        \
        printf("[%s] addr 0x%x reg %d failed![ERROR CODE:0x%lx]\n", #reg, addr, reg, ret);\

#define ITL_PCA9534_WRITE(addr,reg,data) \
    ret = itl_pca9534_out_write_reg(addr, reg, data);\
    if(0 == ret)\
        printf("[%s] write addr 0x%x reg %d = 0x%lx success\n", #reg, addr, reg, data);\
    else        \
        printf("[%s] write addr 0x%x reg %d failed![ERROR CODE:0x%lx]\n", #reg, addr, reg, ret);\

alt_u32 itl_pca9534_out_read_reg(alt_u8 slave_addr, alt_u16 reg, alt_u32 *data);
alt_u32 itl_pca9534_out_write_reg(alt_u8 slave_addr, alt_u16 reg, alt_u32 data);
alt_u32 itl_pca9534_read_reg(alt_u16 reg, alt_u32 *data);
alt_u32 itl_pca9534_write_reg(alt_u16 reg, alt_u32 data);

#endif
